@@ -26,6 +26,10 @@
 #include <termios.h>
 #include <inttypes.h>
 
+#if defined (USE_X11) && defined (USE_XLIB_HACK)
+#include <X11/Xlib.h>
+#endif /* USE_X11 && USE_XLIB_HACK */
+
 #define _GNU_SOURCE
 #include <getopt.h>
 
@@ -41,8 +45,9 @@
   "Options:\n" \
   " -h --help               this help\n" \
   " -p --player <player>    specify the player (mplayer|xine|vlc|gstreamer)\n" \
-  " -a --audio  <audioout>  specify the audio output (alsa|oss|null)\n" \
-  " -g --video  <videoout>  specify the video output (x11|sdl:x11|xv|gl|vdpau|fb|null)\n" \
+  " -a --audio  <audioout>  specify the audio output (alsa|oss|pulse|null)\n" \
+  " -g --video  <videoout>  specify the video output (x11|sdl:x11|xv|gl|vdpau|fb|directfb|vaapi|null)\n" \
+  " -q --quality <level>    specify the picture quality (0|1|2, best to worse)\n" \
   " -v --verbose            increase verbosity\n" \
   "\n" \
   "Default values are dummy player, auto video and auto audio output.\n" \
@@ -54,6 +59,7 @@
   " .   : change loop value and mode\n" \
   " ,   : enable/disable shuffle on the playlist\n" \
   " %%   : write a text on the OSD\n" \
+  " k   : enable/disable OSD\n" \
   " +/- : increase/decrease speed\n" \
   " ]/[ : audio delay +/- 100 ms\n" \
   " 0/9 : increase/decrease volume\n" \
@@ -77,6 +83,7 @@
   " s   : stop the current playback\n" \
   " b   : start the previous stream in the playlist\n" \
   " n   : start the next stream in the playlist\n" \
+  " c   : continue with the next stream accordingly to the playback mode.\n" \
   " r   : remove the current stream of the playlist\n" \
   " t   : remove all streams of the playlist\n" \
   " q   : quit " APPNAME "\n" \
@@ -758,7 +765,8 @@ show_info (player_t *player, mrl_t *mrl)
 
       if (meta)
       {
-        printf (" Meta CD Track %i Name: %s (%i sec)\n", i, meta, length / 1000);
+        printf (" Meta CD Track %i Name: %s (%i sec)\n",
+                i, meta, length / 1000);
         free (meta);
       }
       else
@@ -802,6 +810,7 @@ int
 main (int argc, char **argv)
 {
   player_t *player;
+  player_init_param_t param;
   player_type_t type = PLAYER_TYPE_DUMMY;
   player_vo_t vo = PLAYER_VO_AUTO;
   player_ao_t ao = PLAYER_AO_AUTO;
@@ -811,21 +820,22 @@ main (int argc, char **argv)
   int time_pos, percent_pos;
   float speed = 1.0;
   int loop = 0, shuffle = 0;
-  int visibility = 0;
+  int visibility = 0, osd = 0;
   float ar = 0.0;
   player_loop_t loop_mode = PLAYER_LOOP_DISABLE;
   player_verbosity_level_t verbosity = PLAYER_MSG_WARNING;
   player_pb_t pb_mode = PLAYER_PB_SINGLE;
+  player_quality_level_t quality = PLAYER_QUALITY_NORMAL;
 
-  int c, index, optind_bak, argc_bak, i;
-  char **argv_bak;
-  const char *const short_options = "hvp:a:g:";
+  int c, index;
+  const char *const short_options = "hvp:a:g:q:";
   const struct option long_options [] = {
     {"help",    no_argument,       0, 'h' },
     {"verbose", no_argument,       0, 'v' },
     {"player",  required_argument, 0, 'p' },
     {"audio",   required_argument, 0, 'a' },
     {"video",   required_argument, 0, 'g' },
+    {"quality", required_argument, 0, 'q' },
     {0,         0,                 0,  0  }
   };
 
@@ -861,27 +871,27 @@ main (int argc, char **argv)
       if (!strcmp (optarg, "mplayer"))
 #ifdef HAVE_MPLAYER
         type = PLAYER_TYPE_MPLAYER;
-#else
+#else /* HAVE_MPLAYER */
         printf ("MPlayer not supported, dummy player used instead!\n");
-#endif /* HAVE_MPLAYER */
+#endif /* !HAVE_MPLAYER */
       if (!strcmp (optarg, "xine"))
 #ifdef HAVE_XINE
         type = PLAYER_TYPE_XINE;
-#else
+#else /* HAVE_XINE */
         printf ("xine not supported, dummy player used instead!\n");
-#endif /* HAVE_XINE */
+#endif /* !HAVE_XINE */
       if (!strcmp (optarg, "vlc"))
 #ifdef HAVE_VLC
         type = PLAYER_TYPE_VLC;
-#else
+#else /* HAVE_VLC */
         printf ("VLC not supported, dummy player used instead!\n");
-#endif /* HAVE_VLC */
-      if (!strcmp (optarg, "gstreamer"))
+#endif /* !HAVE_VLC */
+      if (!strcmp (optarg, "gstreamer") || !strcmp (optarg, "gst"))
 #ifdef HAVE_GSTREAMER
         type = PLAYER_TYPE_GSTREAMER;
-#else
+#else /* HAVE_GSTREAMER */
         printf ("GStreamer not supported, dummy player used instead!\n");
-#endif /* HAVE_GSTREAMER */
+#endif /* !HAVE_GSTREAMER */
       break;
 
     case 'a':
@@ -889,6 +899,8 @@ main (int argc, char **argv)
         ao = PLAYER_AO_ALSA;
       else if (!strcmp (optarg, "oss"))
         ao = PLAYER_AO_OSS;
+      else if (!strcmp (optarg, "pulse"))
+        ao = PLAYER_AO_PULSE;
       else if (!strcmp (optarg, "null"))
         ao = PLAYER_AO_NULL;
       break;
@@ -906,28 +918,46 @@ main (int argc, char **argv)
         vo = PLAYER_VO_VDPAU;
       else if (!strcmp (optarg, "fb"))
         vo = PLAYER_VO_FB;
+      else if (!strcmp (optarg, "directfb"))
+        vo = PLAYER_VO_DIRECTFB;
+      else if (!strcmp (optarg, "vaapi"))
+        vo = PLAYER_VO_VAAPI;
       else if (!strcmp (optarg, "null"))
         vo = PLAYER_VO_NULL;
       break;
 
+    case 'q':
+      if (!strcmp (optarg, "0"))
+        quality = PLAYER_QUALITY_NORMAL;
+      else if (!strcmp (optarg, "1"))
+        quality = PLAYER_QUALITY_LOW;
+      else if (!strcmp (optarg, "2"))
+        quality = PLAYER_QUALITY_LOWEST;
+      break;
+
     default:
       printf (TESTPLAYER_HELP);
       return -1;
     }
   }
-  optind_bak = optind;
-  argc_bak = argc;
-  argv_bak = malloc (argc * sizeof (char *));
-  for (i = 0; i < argc; i++)
-    *(argv_bak + i) = strdup (argv[i]);
 
-  player = player_init (type, ao, vo, verbosity, 0, event_cb);
+#if defined (USE_X11) && defined (USE_XLIB_HACK)
+  XInitThreads ();
+#endif /* USE_X11 && USE_XLIB_HACK */
+
+  memset (&param, 0, sizeof (player_init_param_t));
+  param.ao       = ao;
+  param.vo       = vo;
+  param.event_cb = event_cb;
+  param.quality  = quality;
+
+  player = player_init (type, verbosity, &param);
 
   if (!player)
     return -1;
 
   /* these arguments are files */
-  if (optind_bak < argc_bak)
+  if (optind < argc)
   {
     do
     {
@@ -935,19 +965,16 @@ main (int argc, char **argv)
       mrl_resource_local_args_t *args;
 
       args = calloc (1, sizeof (mrl_resource_local_args_t));
-      args->location = strdup (argv_bak[optind_bak]);
+      args->location = strdup (argv[optind]);
 
       mrl = mrl_new (player, MRL_RESOURCE_FILE, args);
       if (!mrl)
         continue;
-      printf (" > %s added to the playlist!\n", argv_bak[optind_bak]);
+      printf (" > %s added to the playlist!\n", argv[optind]);
       player_mrl_append (player, mrl, PLAYER_MRL_ADD_QUEUE);
-    } while (++optind_bak < argc_bak);
+    } while (++optind < argc);
     putchar ('\n');
   }
-  for (i = 0; i < argc; i++)
-    free (*(argv_bak + i));
-  free (argv_bak);
 
   printf (TESTPLAYER_COMMANDS);
 
@@ -1001,6 +1028,10 @@ main (int argc, char **argv)
       printf ("OSD SHOW TEXT (5s): %s\n", text);
       break;
     }
+    case 'k':
+      osd = !osd;
+      player_osd_state (player, osd);
+      break;
     case '+':
       speed += 0.1;
       if (speed > 100.0)
@@ -1046,11 +1077,11 @@ main (int argc, char **argv)
       printf ("VOLUME %i\n", volume);
       break;
     case '1':   /* 5s backward */
-      player_playback_seek (player, -5, PLAYER_PB_SEEK_RELATIVE);
+      player_playback_seek (player, -5000, PLAYER_PB_SEEK_RELATIVE);
       printf ("SEEK -5 sec.\n");
       break;
     case '2':   /* 5s forward */
-      player_playback_seek (player, 5, PLAYER_PB_SEEK_RELATIVE);
+      player_playback_seek (player, 5000, PLAYER_PB_SEEK_RELATIVE);
       printf ("SEEK +5 sec.\n");
       break;
     case '3':
@@ -1127,6 +1158,10 @@ main (int argc, char **argv)
       player_mrl_next (player);
       printf ("NEXT STREAM\n");
       break;
+    case 'c':   /* continue with the next stream accordingly to the pb mode. */
+      player_mrl_continue (player);
+      printf ("CONTINUE\n");
+      break;
     case 'o':   /* pause the current playback */
       player_playback_pause (player);
       printf ("PAUSE\n");
