@@ -50,7 +50,7 @@
 #define MODULE_NAME "player"
 
 static int
-player_event_cb (void *data, int e, void *data_cb)
+player_event_cb (void *data, int e)
 {
   int res = 0;
   player_t *player = data;
@@ -68,7 +68,7 @@ player_event_cb (void *data, int e, void *data_cb)
   if (player->event_cb)
   {
     pl_supervisor_callback_in (player, pthread_self ());
-    res = player->event_cb (e, data_cb);
+    res = player->event_cb (e, player->user_data);
     pl_supervisor_callback_out (player);
   }
 
@@ -89,6 +89,12 @@ player_event_cb (void *data, int e, void *data_cb)
   return res;
 }
 
+unsigned int
+libplayer_version (void)
+{
+  return LIBPLAYER_VERSION_INT;
+}
+
 /***************************************************************************/
 /*                                                                         */
 /* Player (Un)Initialization                                               */
@@ -96,9 +102,8 @@ player_event_cb (void *data, int e, void *data_cb)
 /***************************************************************************/
 
 player_t *
-player_init (player_type_t type, player_ao_t ao, player_vo_t vo,
-             player_verbosity_level_t verbosity, unsigned long winid,
-             int (*event_cb) (player_event_t e, void *data))
+player_init (player_type_t type,
+             player_verbosity_level_t verbosity, player_init_param_t *param)
 {
   player_t *player = NULL;
   init_status_t res = PLAYER_INIT_ERROR;
@@ -110,18 +115,25 @@ player_init (player_type_t type, player_ao_t ao, player_vo_t vo,
   pthread_cond_t *sv_cond;
   pthread_mutex_t *sv_mutex;
 
-  player = calloc (1, sizeof (player_t));
+  player = PCALLOC (player_t, 1);
   if (!player)
     return NULL;
 
-  player->type      = type;
-  player->verbosity = verbosity;
-  player->state     = PLAYER_STATE_IDLE;
-  player->ao        = ao;
-  player->vo        = vo;
-  player->winid     = winid;
-  player->event_cb  = event_cb;
-  player->playlist  = pl_playlist_new (0, 0, PLAYER_LOOP_DISABLE);
+  player->type        = type;
+  player->verbosity   = verbosity;
+  player->state       = PLAYER_STATE_IDLE;
+  player->playlist    = pl_playlist_new (0, 0, PLAYER_LOOP_DISABLE);
+
+  if (param)
+  {
+    player->ao          = param->ao;
+    player->vo          = param->vo;
+    player->winid       = param->winid;
+    player->x11_display = param->display;
+    player->event_cb    = param->event_cb;
+    player->user_data   = param->data;
+    player->quality     = param->quality;
+  }
 
   pthread_mutex_init (&player->mutex_verb, NULL);
 
@@ -236,8 +248,8 @@ player_uninit (player_t *player)
 
   pl_playlist_free (player->playlist);
   pthread_mutex_destroy (&player->mutex_verb);
-  free (player->funcs);
-  free (player);
+  PFREE (player->funcs);
+  PFREE (player);
 }
 
 void
@@ -351,6 +363,18 @@ player_mrl_next (player_t *player)
                       SV_FUNC_PLAYER_MRL_NEXT, NULL, NULL);
 }
 
+void
+player_mrl_continue (player_t *player)
+{
+  pl_log (player, PLAYER_MSG_VERBOSE, MODULE_NAME, __FUNCTION__);
+
+  if (!player)
+    return;
+
+  pl_supervisor_send (player, SV_MODE_WAIT_FOR_END,
+                      SV_FUNC_PLAYER_MRL_NEXT_PLAY, NULL, NULL);
+}
+
 /***************************************************************************/
 /*                                                                         */
 /* Player tuning & properties                                              */
@@ -500,6 +524,18 @@ player_osd_show_text (player_t *player,
                       SV_FUNC_PLAYER_OSD_SHOW_TEXT, &in, NULL);
 }
 
+void
+player_osd_state (player_t *player, int value)
+{
+  pl_log (player, PLAYER_MSG_VERBOSE, MODULE_NAME, __FUNCTION__);
+
+  if (!player)
+    return;
+
+  pl_supervisor_send (player, SV_MODE_WAIT_FOR_END,
+                      SV_FUNC_PLAYER_OSD_STATE, &value, NULL);
+}
+
 /***************************************************************************/
 /*                                                                         */
 /* Playback related controls                                               */
@@ -725,18 +761,6 @@ player_audio_next (player_t *player)
 /*                                                                         */
 /***************************************************************************/
 
-void
-player_video_set_fullscreen (player_t *player, int value)
-{
-  pl_log (player, PLAYER_MSG_VERBOSE, MODULE_NAME, __FUNCTION__);
-
-  if (!player)
-    return;
-
-  pl_supervisor_send (player, SV_MODE_WAIT_FOR_END,
-                      SV_FUNC_PLAYER_VO_SET_FS, &value, NULL);
-}
-
 void
 player_video_set_aspect (player_t *player, player_video_aspect_t aspect,
                          int8_t value, int absolute)
