@@ -67,13 +67,34 @@ extern "C" {
 #endif
 #endif
 
+#define PL_STRINGIFY(s) #s
+#define PL_TOSTRING(s) PL_STRINGIFY(s)
+
+#define PL_VERSION_INT(a, b, c) (a << 16 | b << 8 | c)
+#define PL_VERSION_DOT(a, b, c) a ##.## b ##.## c
+#define PL_VERSION(a, b, c) PL_VERSION_DOT(a, b, c)
+
+#define LIBPLAYER_VERSION_MAJOR  2
+#define LIBPLAYER_VERSION_MINOR  0
+#define LIBPLAYER_VERSION_MICRO  1
+
+#define LIBPLAYER_VERSION_INT PL_VERSION_INT(LIBPLAYER_VERSION_MAJOR, \
+                                             LIBPLAYER_VERSION_MINOR, \
+                                             LIBPLAYER_VERSION_MICRO)
+#define LIBPLAYER_VERSION     PL_VERSION(LIBPLAYER_VERSION_MAJOR, \
+                                         LIBPLAYER_VERSION_MINOR, \
+                                         LIBPLAYER_VERSION_MICRO)
+#define LIBPLAYER_VERSION_STR PL_TOSTRING(LIBPLAYER_VERSION)
+#define LIBPLAYER_BUILD       LIBPLAYER_VERSION_INT
+
 #include <inttypes.h>
 #include <sys/types.h>
 
-#define LIBPLAYER_VERSION_MAJOR 1
-#define LIBPLAYER_VERSION_MINOR 0
-#define LIBPLAYER_VERSION_MICRO 0
-#define LIBPLAYER_VERSION "1.0.0"
+/**
+ * \brief Return LIBPLAYER_VERSION_INT constant.
+ */
+unsigned int libplayer_version (void);
+
 
 /***************************************************************************/
 /*                                                                         */
@@ -100,22 +121,25 @@ typedef enum player_type {
 
 /** \brief Player video outputs. */
 typedef enum player_vo {
+  PLAYER_VO_AUTO = 0,
   PLAYER_VO_NULL,
-  PLAYER_VO_AUTO,
   PLAYER_VO_X11,
   PLAYER_VO_X11_SDL,
   PLAYER_VO_XV,
   PLAYER_VO_GL,
   PLAYER_VO_FB,
-  PLAYER_VO_VDPAU
+  PLAYER_VO_DIRECTFB,
+  PLAYER_VO_VDPAU,
+  PLAYER_VO_VAAPI,
 } player_vo_t;
 
 /** \brief Player audio outputs. */
 typedef enum player_ao {
+  PLAYER_AO_AUTO = 0,
   PLAYER_AO_NULL,
-  PLAYER_AO_AUTO,
   PLAYER_AO_ALSA,
-  PLAYER_AO_OSS
+  PLAYER_AO_OSS,
+  PLAYER_AO_PULSE,
 } player_ao_t;
 
 /** \brief Player events. */
@@ -139,6 +163,40 @@ typedef enum {
   PLAYER_MSG_CRITICAL,      /* prevents lib from working */
 } player_verbosity_level_t;
 
+typedef enum {
+  PLAYER_QUALITY_NORMAL,    /* normal picture quality */
+  PLAYER_QUALITY_LOW,       /* slightly degraded picture for fastest playback */
+  PLAYER_QUALITY_LOWEST,    /* degraded picture, suitable for low-end CPU */
+} player_quality_level_t;
+
+/** \brief Parameters for player_init() .*/
+typedef struct player_init_param_s {
+  /** Audio output driver. */
+  player_ao_t ao;
+  /** Video output driver. */
+  player_vo_t vo;
+  /** Window ID to attach the video (X Window). */
+  uint32_t winid;
+
+  /** Public event callback. */
+  int (*event_cb) (player_event_t e, void *data);
+  /** User data for event callback. */
+  void *data;
+
+  /**
+   * Display to use with X11 video outputs.
+   *
+   * The string has to follow the same rules that the DISPLAY environment
+   * variable. If \p display is NULL, then the environment variable is
+   * considered.
+   */
+  const char *display;
+
+  /** Picture decoding quality. */
+  player_quality_level_t quality;
+
+} player_init_param_t;
+
 /**
  * \name Player (Un)Initialization.
  * @{
@@ -150,21 +208,22 @@ typedef enum {
  * Multiple player controllers can be initialized with any wrappers.
  * The same Window ID can be used to attach their video.
  *
+ * For a description of each parameters supported by this function:
+ * \see ::player_init_param_t
+ *
+ * When a parameter in \p param is 0 (or NULL), its default value is used.
+ * If \p param is NULL, then all default values are forced for all parameters.
+ *
  * Wrappers supported (even partially):
  *  GStreamer, MPlayer, VLC, xine
  *
- * \warning MT-Safe in multithreaded applications (see \ref mtlevel).
  * \param[in] type        Type of wrapper to load.
- * \param[in] ao          Audio output driver to use.
- * \param[in] vo          Video output driver to use.
  * \param[in] verbosity   Level of verbosity to set.
- * \param[in] winid       WinID to attach the video (X Window), 0 to disable.
- * \param[in] event_cb    Public callback, NULL to disable.
+ * \param[in] param       Parameters, NULL for default values.
  * \return Player controller, NULL otherwise.
  */
-player_t *player_init (player_type_t type, player_ao_t ao, player_vo_t vo,
-                       player_verbosity_level_t verbosity, unsigned long winid,
-                       int (*event_cb) (player_event_t e, void *data));
+player_t *player_init (player_type_t type, player_verbosity_level_t verbosity,
+                       player_init_param_t *param);
 
 /**
  * \brief Uninitialization of a player controller.
@@ -229,7 +288,7 @@ typedef enum mrl_resource {
 
   /* Local Streams */
   MRL_RESOURCE_FIFO,        /*  NO           NO           NO           NO   */
-  MRL_RESOURCE_FILE,        /*  NO           YES          YES          YES  */
+  MRL_RESOURCE_FILE,        /*  YES          YES          YES          YES  */
   MRL_RESOURCE_STDIN,       /*  NO           NO           NO           NO   */
 
   /* Audio CD */
@@ -237,12 +296,12 @@ typedef enum mrl_resource {
   MRL_RESOURCE_CDDB,        /*  NO           YES          NO           NO   */
 
   /* Video discs */
-  MRL_RESOURCE_DVD,         /*  NO           YES          NO           YES  */
-  MRL_RESOURCE_DVDNAV,      /*  NO           YES          NO           YES  */
+  MRL_RESOURCE_DVD,         /*  NO           YES          YES          YES  */
+  MRL_RESOURCE_DVDNAV,      /*  NO           YES          YES          YES  */
   MRL_RESOURCE_VCD,         /*  NO           YES          NO           NO   */
 
   /* Radio/Television */
-  MRL_RESOURCE_DVB,         /*  NO           NO           NO           NO   */
+  MRL_RESOURCE_DVB,         /*  NO           YES          NO           NO   */
   MRL_RESOURCE_PVR,         /*  NO           NO           NO           NO   */
   MRL_RESOURCE_RADIO,       /*  NO           YES          NO           NO   */
   MRL_RESOURCE_TV,          /*  NO           YES          NO           NO   */
@@ -250,20 +309,20 @@ typedef enum mrl_resource {
 
   /* Network Streams */
   MRL_RESOURCE_FTP,         /*  NO           YES          YES          NO   */
-  MRL_RESOURCE_HTTP,        /*  NO           YES          YES          NO   */
-  MRL_RESOURCE_MMS,         /*  NO           YES          YES          NO   */
+  MRL_RESOURCE_HTTP,        /*  NO           YES          YES          YES  */
+  MRL_RESOURCE_MMS,         /*  NO           YES          YES          YES  */
   MRL_RESOURCE_NETVDR,      /*  NO           NO           NO           YES  */
-  MRL_RESOURCE_RTP,         /*  NO           YES          YES          NO   */
+  MRL_RESOURCE_RTP,         /*  NO           YES          YES          YES  */
   MRL_RESOURCE_RTSP,        /*  NO           YES          YES          NO   */
   MRL_RESOURCE_SMB,         /*  NO           YES          YES          NO   */
-  MRL_RESOURCE_TCP,         /*  NO           NO           NO           NO   */
-  MRL_RESOURCE_UDP,         /*  NO           YES          YES          NO   */
+  MRL_RESOURCE_TCP,         /*  NO           NO           NO           YES  */
+  MRL_RESOURCE_UDP,         /*  NO           YES          YES          YES  */
   MRL_RESOURCE_UNSV,        /*  NO           YES          YES          NO   */
 } mrl_resource_t;
 
 /** \brief Arguments for local streams. */
 typedef struct mrl_resource_local_args_s {
-  char *location;           /*  NO           YES          YES          YES  */
+  char *location;           /*  YES          YES          YES          YES  */
   int playlist;             /*  NO           NO           NO           NO   */
 } mrl_resource_local_args_t;
 
@@ -277,12 +336,12 @@ typedef struct mrl_resource_cd_args_s {
 
 /** \brief Arguments for video discs. */
 typedef struct mrl_resource_videodisc_args_s {
-  char *device;             /*  NO           YES          NO           YES  */
+  char *device;             /*  NO           YES          YES          YES  */
   uint8_t speed;            /*  NO           NO           NO           NO   */
-  uint8_t angle;            /*  NO           YES          NO           NO   */
-  uint8_t title_start;      /*  NO           YES          NO           YES  */
+  uint8_t angle;            /*  NO           YES          YES          NO   */
+  uint8_t title_start;      /*  NO           YES          YES          YES  */
   uint8_t title_end;        /*  NO           YES          NO           NO   */
-  uint8_t chapter_start;    /*  NO           NO           NO           NO   */
+  uint8_t chapter_start;    /*  NO           NO           YES          NO   */
   uint8_t chapter_end;      /*  NO           NO           NO           NO   */
   uint8_t track_start;      /*  NO           YES          NO           NO   */
   uint8_t track_end;        /*  NO           NO           NO           NO   */
@@ -374,6 +433,10 @@ typedef enum mrl_properties_type {
  *
  * This function can be slow when the stream is not (fastly) reachable.
  *
+ * The argument \p args and the strings provided with \p args must be
+ * allocated dynamically. The pointers are freed by libplayer when a mrl
+ * is no longer available.
+ *
  * \warning MT-Safe in multithreaded applications (see \ref mtlevel).
  * \param[in] player      Player controller.
  * \param[in] res         Resource type.
@@ -760,6 +823,17 @@ void player_mrl_previous (player_t *player);
  */
 void player_mrl_next (player_t *player);
 
+/**
+ * \brief Go to the next MRL object accordingly to the loop and shuffle.
+ *
+ * The behaviour is the same that player_mrl_next() if the 'loop' or the 'shuffle'
+ * is not enabled and the playback mode is not AUTO.
+ *
+ * \warning MT-Safe in multithreaded applications (see \ref mtlevel).
+ * \param[in] player      Player controller.
+ */
+void player_mrl_continue (player_t *player);
+
 /**
  * @}
  */
@@ -932,6 +1006,21 @@ void player_x_window_set_properties (player_t *player,
 void player_osd_show_text (player_t *player,
                            const char *text, int x, int y, int duration);
 
+/**
+ * \brief Enable/disable On-screen Display.
+ *
+ * With the MPlayer wrapper, this function must be called after every
+ * player_playback_start() if OSD must be disabled.
+ *
+ * Wrappers supported (even partially):
+ *  MPlayer
+ *
+ * \warning MT-Safe in multithreaded applications (see \ref mtlevel).
+ * \param[in] player      Player controller.
+ * \param[in] value       Different of 0 to enable.
+ */
+void player_osd_state (player_t *player, int value);
+
 /**
  * @}
  */
@@ -1013,7 +1102,7 @@ void player_playback_pause (player_t *player);
  *
  * \warning MT-Safe in multithreaded applications (see \ref mtlevel).
  * \param[in] player      Player controller.
- * \param[in] value       Value for seeking (second or percent).
+ * \param[in] value       Value for seeking (millisecond or percent).
  * \param[in] seek        Seeking mode.
  */
 void player_playback_seek (player_t *player, int value, player_pb_seek_t seek);
@@ -1037,7 +1126,7 @@ void player_playback_seek_chapter (player_t *player, int value, int absolute);
  * This function can't be used to play in backward.
  *
  * Wrappers supported (even partially):
- *  MPlayer, VLC
+ *  MPlayer, xine, VLC
  *
  * \warning MT-Safe in multithreaded applications (see \ref mtlevel).
  * \param[in] player      Player controller.
@@ -1192,21 +1281,6 @@ typedef enum player_video_aspect {
  * @{
  */
 
-/**
- * \brief Set video in fullscreen.
- *
- * By default the video is always in fullscreen. To work with a window,
- * use winid parameter in player_init().
- *
- * Wrappers supported (even partially):
- *  MPlayer, VLC
- *
- * \warning MT-Safe in multithreaded applications (see \ref mtlevel).
- * \param[in] player      Player controller.
- * \param[in] value       Different of 0 to set fullscreen.
- */
-void player_video_set_fullscreen (player_t *player, int value);
-
 /**
  * \brief Set video aspect.
  *
@@ -1239,7 +1313,7 @@ void player_video_set_panscan (player_t *player, int8_t value, int absolute);
  * \brief Set video aspect ratio.
  *
  * Wrappers supported (even partially):
- *  MPlayer, VLC
+ *  MPlayer, VLC, xine
  *
  * \warning MT-Safe in multithreaded applications (see \ref mtlevel).
  * \param[in] player      Player controller.
@@ -1605,7 +1679,7 @@ void player_radio_channel_next (player_t *player);
 
 /** \brief Player VDR commands. */
 typedef enum player_vdr {
-  PLAYER_VDR_UP,
+  PLAYER_VDR_UP = 0,
   PLAYER_VDR_DOWN,
   PLAYER_VDR_LEFT,
   PLAYER_VDR_RIGHT,
