@@ -196,11 +196,17 @@ player_sv_mrl_next_play (player_t *player)
   if (!player)
     return;
 
+  if (player->pb_mode != PLAYER_PB_AUTO)
+  {
+    player_sv_mrl_next (player);
+    return;
+  }
+
   player_sv_playback_stop (player);
 
   if (!pl_playlist_next_play (player->playlist))
   {
-    player_event_send (player, PLAYER_EVENT_PLAYLIST_FINISHED, NULL);
+    player_event_send (player, PLAYER_EVENT_PLAYLIST_FINISHED);
     return;
   }
 
@@ -316,7 +322,10 @@ player_sv_x_window_set_properties (player_t *player,
 {
 #ifdef USE_X11
   int f = 0;
-#endif /* USE_X11 */
+#else /* USE_X11 */
+  player = NULL;
+  x = y = w = h = flags = 0;
+#endif /* !USE_X11 */
 
   pl_log (player, PLAYER_MSG_VERBOSE, MODULE_NAME, __FUNCTION__);
 
@@ -363,6 +372,18 @@ player_sv_osd_show_text (player_t *player,
   PLAYER_FUNCS (osd_show_text, text, x, y, duration)
 }
 
+void
+player_sv_osd_state (player_t *player, int value)
+{
+  pl_log (player, PLAYER_MSG_VERBOSE, MODULE_NAME, __FUNCTION__);
+
+  if (!player)
+    return;
+
+  /* player specific osd_state() */
+  PLAYER_FUNCS (osd_state, value)
+}
+
 /***************************************************************************/
 /*                                                                         */
 /* Playback related controls                                               */
@@ -426,7 +447,7 @@ player_sv_playback_start (player_t *player)
   player->state = PLAYER_STATE_RUNNING;
 
   /* notify front-end */
-  player_event_send (player, PLAYER_EVENT_PLAYBACK_START, NULL);
+  player_event_send (player, PLAYER_EVENT_PLAYBACK_START);
 }
 
 void
@@ -446,7 +467,7 @@ player_sv_playback_stop (player_t *player)
   player->state = PLAYER_STATE_IDLE;
 
   /* notify front-end */
-  player_event_send (player, PLAYER_EVENT_PLAYBACK_STOP, NULL);
+  player_event_send (player, PLAYER_EVENT_PLAYBACK_STOP);
 }
 
 void
@@ -472,12 +493,12 @@ player_sv_playback_pause (player_t *player)
   if (player->state == PLAYER_STATE_RUNNING)
   {
     player->state = PLAYER_STATE_PAUSE;
-    player_event_send (player, PLAYER_EVENT_PLAYBACK_PAUSE, NULL);
+    player_event_send (player, PLAYER_EVENT_PLAYBACK_PAUSE);
   }
   else
   {
     player->state = PLAYER_STATE_RUNNING;
-    player_event_send (player, PLAYER_EVENT_PLAYBACK_UNPAUSE, NULL);
+    player_event_send (player, PLAYER_EVENT_PLAYBACK_UNPAUSE);
   }
 }
 
@@ -633,18 +654,6 @@ player_sv_audio_next (player_t *player)
 /*                                                                         */
 /***************************************************************************/
 
-void
-player_sv_video_set_fullscreen (player_t *player, int value)
-{
-  pl_log (player, PLAYER_MSG_VERBOSE, MODULE_NAME, __FUNCTION__);
-
-  if (!player)
-    return;
-
-  /* player specific video_set_fs() */
-  PLAYER_FUNCS (video_set_fs, value)
-}
-
 void
 player_sv_video_set_aspect (player_t *player, player_video_aspect_t aspect,
                             int8_t value, int absolute)
