@@ -532,6 +532,17 @@ supervisor_player_osd_show_text (player_t *player,
                            input->text, input->x, input->y, input->duration);
 }
 
+static void
+supervisor_player_osd_state (player_t *player, void *in, pl_unused void *out)
+{
+  int *input = in;
+
+  if (!player || !in)
+    return;
+
+  player_sv_osd_state (player, *input);
+}
+
 /************************ Playback related controls **************************/
 
 static void
@@ -708,18 +719,6 @@ supervisor_player_ao_next (player_t *player,
 
 /************************** Video related controls ***************************/
 
-static void
-supervisor_player_vo_set_fs (player_t *player,
-                             void *in, pl_unused void *out)
-{
-  int *input = in;
-
-  if (!player || !in)
-    return;
-
-  player_sv_video_set_fullscreen (player, *input);
-}
-
 static void
 supervisor_player_vo_set_aspect (player_t *player,
                                  void *in, pl_unused void *out)
@@ -1052,6 +1051,7 @@ static void (*g_supervisor_funcs[]) (player_t *player, void *in, void *out) = {
   [SV_FUNC_PLAYER_SET_MOUSE_POS]         = supervisor_player_set_mouse_pos,
   [SV_FUNC_PLAYER_X_WINDOW_SET_PROPS]    = supervisor_player_x_window_set_props,
   [SV_FUNC_PLAYER_OSD_SHOW_TEXT]         = supervisor_player_osd_show_text,
+  [SV_FUNC_PLAYER_OSD_STATE]             = supervisor_player_osd_state,
 
   /* Playback related controls */
   [SV_FUNC_PLAYER_PB_GET_STATE]          = supervisor_player_pb_get_state,
@@ -1073,7 +1073,6 @@ static void (*g_supervisor_funcs[]) (player_t *player, void *in, void *out) = {
   [SV_FUNC_PLAYER_AO_NEXT]               = supervisor_player_ao_next,
 
   /* Video related controls */
-  [SV_FUNC_PLAYER_VO_SET_FS]             = supervisor_player_vo_set_fs,
   [SV_FUNC_PLAYER_VO_SET_ASPECT]         = supervisor_player_vo_set_aspect,
   [SV_FUNC_PLAYER_VO_SET_PANSCAN]        = supervisor_player_vo_set_panscan,
   [SV_FUNC_PLAYER_VO_SET_AR]             = supervisor_player_vo_set_ar,
@@ -1227,7 +1226,7 @@ thread_supervisor (void *arg)
     in = data->in;
     out = data->out;
     mode = data->mode;
-    free (data);
+    PFREE (data);
 
     supervisor_sync_catch (supervisor);
 
@@ -1365,7 +1364,7 @@ pl_supervisor_send (player_t *player, supervisor_mode_t mode,
     sem_wait (&supervisor->sem_ctl);
   else if (res)
   {
-    free (data);
+    PFREE (data);
     pl_log (player, PLAYER_MSG_ERROR,
             MODULE_NAME, "error on queue? no sense :(");
   }
@@ -1379,14 +1378,14 @@ pl_supervisor_new (void)
 {
   supervisor_t *supervisor;
 
-  supervisor = calloc (1, sizeof (supervisor_t));
+  supervisor = PCALLOC (supervisor_t, 1);
   if (!supervisor)
     return NULL;
 
   supervisor->queue = pl_fifo_queue_new ();
   if (!supervisor->queue)
   {
-    free (supervisor);
+    PFREE (supervisor);
     return NULL;
   }
 
@@ -1472,6 +1471,6 @@ pl_supervisor_uninit (player_t *player)
 
   pthread_mutex_destroy (&supervisor->mutex_cb);
 
-  free (supervisor);
+  PFREE (supervisor);
   player->supervisor = NULL;
 }
