@@ -1,6 +1,6 @@
 /*
  * GeeXboX libplayer: a multimedia A/V abstraction layer API.
- * Copyright (C) 2006 Benjamin Zores <ben@geexbox.org>
+ * Copyright (C) 2006-2009 Benjamin Zores <ben@geexbox.org>
  *
  * This file is part of libplayer.
  *
@@ -35,6 +35,10 @@
 #include "parse_utils.h"
 #include "wrapper_vlc.h"
 
+#ifdef USE_X11
+#include "x11_common.h"
+#endif /* USE_X11 */
+
 #define MODULE_NAME "vlc"
 
 #define WAIT_PERIOD    1000 /* in micro-seconds */
@@ -44,7 +48,6 @@
 typedef struct vlc_s {
   libvlc_instance_t *core;
   libvlc_media_player_t *mp;
-  libvlc_exception_t ex;
 } vlc_t;
 
 static const libvlc_event_type_t mp_events[] = {
@@ -71,56 +74,78 @@ vlc_event_callback (const libvlc_event_t *ev, void *data)
   type = ev->type;
   switch (type)
   {
-  case libvlc_MediaPlayerPlaying:
-    pl_log (player, PLAYER_MSG_INFO,
-            MODULE_NAME, "Playback of stream has started");
-    player_event_send (player, PLAYER_EVENT_PLAYBACK_START, NULL);
-    break;
-
-  case libvlc_MediaPlayerPaused:
-    pl_log (player, PLAYER_MSG_INFO,
-            MODULE_NAME, "Playback of stream has been paused");
-    player_event_send (player, PLAYER_EVENT_PLAYBACK_PAUSE, NULL);
-    break;
-
   case libvlc_MediaPlayerEndReached:
     pl_log (player, PLAYER_MSG_INFO,
             MODULE_NAME, "Playback of stream has ended");
-    player_event_send (player, PLAYER_EVENT_PLAYBACK_FINISHED, NULL);
+    player_event_send (player, PLAYER_EVENT_PLAYBACK_FINISHED);
+
+#ifdef USE_X11
+    pl_x11_unmap (player);
+#endif /* USE_X11 */
     break;
 
+  case libvlc_MediaPlayerPlaying:
+  case libvlc_MediaPlayerPaused:
   case libvlc_MediaPlayerStopped:
-    pl_log (player, PLAYER_MSG_INFO,
-            MODULE_NAME, "Playback of stream has stopped");
-    player_event_send (player, PLAYER_EVENT_PLAYBACK_STOP, NULL);
     break;
 
   default:
     pl_log (player, PLAYER_MSG_INFO, MODULE_NAME,
             "Unknown event received: %s", libvlc_event_type_name (type));
-    player_event_send (player, PLAYER_EVENT_UNKNOWN, NULL);
     break;
   }
 }
 
-static void
-vlc_check_exception (player_t *player)
+static char *
+vlc_resource_get_uri_dvd (const char *protocol,
+                          mrl_resource_videodisc_args_t *args)
 {
-  vlc_t *vlc;
+  char *uri;
+  char title_start[8] = "";
+  char chapter_start[8] = "";
+  char angle[8] = "";
+  size_t size;
 
-  if (!player)
-    return;
+  if (!args || !protocol)
+    return NULL;
 
-  vlc = (vlc_t *) player->priv;
-  if (!vlc)
-    return;
+  size = strlen (protocol);
 
-  if (!libvlc_exception_raised (&vlc->ex))
-    return;
+  if (args->device)
+    size += strlen (args->device);
 
-  pl_log (player, PLAYER_MSG_WARNING,
-          MODULE_NAME, libvlc_exception_get_message (&vlc->ex));
-  libvlc_exception_clear (&vlc->ex);
+  if (args->title_start)
+  {
+    /* dvd://@title */
+    snprintf (title_start, sizeof (title_start), "@%u", args->title_start);
+    size += strlen (title_start);
+  }
+  if (args->chapter_start)
+  {
+    /* dvd://@:chapter */
+    snprintf (chapter_start, sizeof (chapter_start), "%s:%u",
+              args->title_start ? "" : "@",
+              args->chapter_start);
+    size += strlen (chapter_start);
+  }
+  if (args->angle)
+  {
+    /* dvd://@::angle */
+    snprintf (angle, sizeof (angle), "%s%s:%u",
+              args->title_start   ? "" : "@",
+              args->chapter_start ? "" : ":",
+              args->angle);
+    size += strlen (angle);
+  }
+
+  size++;
+  uri = malloc (size);
+  if (uri)
+    snprintf (uri, size, "%s%s%s%s%s",
+              protocol, args->device ? args->device : "",
+              title_start, chapter_start, angle);
+
+  return uri;
 }
 
 static char *
@@ -159,7 +184,7 @@ vlc_resource_get_uri_network (const char *protocol,
   if (uri)
     snprintf (uri, size, "%s%s%s", protocol, at, host_file);
 
-  free (host_file);
+  PFREE (host_file);
 
   return uri;
 }
@@ -171,18 +196,9 @@ vlc_resource_get_uri (mrl_t *mrl)
     /* Local Streams */
     [MRL_RESOURCE_FILE]     = "file://",
 
-    /* Audio CD */
-    [MRL_RESOURCE_CDDA]     = "cdda://",
-    [MRL_RESOURCE_CDDB]     = "cddb://",
-
     /* Video discs */
-    [MRL_RESOURCE_DVD]      = "dvd://",
-    [MRL_RESOURCE_DVDNAV]   = "dvdnav://",
-    [MRL_RESOURCE_VCD]      = "vcd://",
-
-    /* Radio/Television */
-    [MRL_RESOURCE_RADIO]    = "radio://",
-    [MRL_RESOURCE_TV]       = "tv://",
+    [MRL_RESOURCE_DVD]      = "dvdsimple://",
+    [MRL_RESOURCE_DVDNAV]   = "dvd://",
 
     /* Network Streams */
     [MRL_RESOURCE_FTP]      = "ftp://",
@@ -212,6 +228,10 @@ vlc_resource_get_uri (mrl_t *mrl)
     return strdup (args->location);
   }
 
+  case MRL_RESOURCE_DVD:    /* dvdsimple://device@title:chapter:angle */
+  case MRL_RESOURCE_DVDNAV: /* dvd://device@title:chapter:angle       */
+    return vlc_resource_get_uri_dvd (protocols[mrl->resource], mrl->priv);
+
   case MRL_RESOURCE_FTP:  /* ftp://username:password@url   */
   case MRL_RESOURCE_HTTP: /* http://username:password@url  */
   case MRL_RESOURCE_MMS:  /* mms://username:password@url   */
@@ -234,16 +254,15 @@ vlc_resource_get_uri (mrl_t *mrl)
 /*****************************************************************************/
 
 static void
-vlc_identify_metadata (mrl_t *mrl, libvlc_media_player_t *mp,
-                       libvlc_exception_t *ex)
+vlc_identify_metadata (mrl_t *mrl, libvlc_media_player_t *mp)
 {
   mrl_metadata_t *meta;
   libvlc_media_t *media;
 
-  if (!mp || !ex || !mrl || !mrl->meta)
+  if (!mp || !mrl || !mrl->meta)
     return;
 
-  media = libvlc_media_player_get_media (mp, ex);
+  media = libvlc_media_player_get_media (mp);
   if (!media)
     return;
 
@@ -251,49 +270,84 @@ vlc_identify_metadata (mrl_t *mrl, libvlc_media_player_t *mp,
   if (!meta)
     return;
 
-  meta->title   = libvlc_media_get_meta (media, libvlc_meta_Title,       ex);
-  meta->artist  = libvlc_media_get_meta (media, libvlc_meta_Artist,      ex);
-  meta->genre   = libvlc_media_get_meta (media, libvlc_meta_Genre,       ex);
-  meta->album   = libvlc_media_get_meta (media, libvlc_meta_Album,       ex);
-  meta->year    = libvlc_media_get_meta (media, libvlc_meta_Date,        ex);
-  meta->track   = libvlc_media_get_meta (media, libvlc_meta_TrackNumber, ex);
-  meta->comment = libvlc_media_get_meta (media, libvlc_meta_Description, ex);
+  meta->title   = libvlc_media_get_meta (media, libvlc_meta_Title);
+  meta->artist  = libvlc_media_get_meta (media, libvlc_meta_Artist);
+  meta->genre   = libvlc_media_get_meta (media, libvlc_meta_Genre);
+  meta->album   = libvlc_media_get_meta (media, libvlc_meta_Album);
+  meta->year    = libvlc_media_get_meta (media, libvlc_meta_Date);
+  meta->track   = libvlc_media_get_meta (media, libvlc_meta_TrackNumber);
+  meta->comment = libvlc_media_get_meta (media, libvlc_meta_Description);
 }
 
 static void
-vlc_identify_audio (pl_unused mrl_t *mrl, pl_unused libvlc_media_player_t *mp,
-                    pl_unused libvlc_exception_t *ex)
+vlc_identify_audio (mrl_t *mrl,
+                    libvlc_media_player_t *mp, libvlc_media_track_info_t *es)
 {
-  /* VLC API is not yet complete enough to retrieve these info */
+  mrl_properties_audio_t *audio;
+
+  if (!mrl || !mrl->prop || !mp || !es)
+    return;
+
+  if (!mrl->prop->audio)
+    mrl->prop->audio = mrl_properties_audio_new ();
+
+  audio = mrl->prop->audio;
+
+  audio->bitrate  = es->u.audio.i_rate;
+  audio->channels = es->u.audio.i_channels;
 }
 
 static void
-vlc_identify_video (mrl_t *mrl, libvlc_media_player_t *mp,
-                    libvlc_exception_t *ex)
+vlc_identify_video (mrl_t *mrl,
+                    libvlc_media_player_t *mp, libvlc_media_track_info_t *es)
 {
   mrl_properties_video_t *video;
   libvlc_track_description_t *tracks, *t;
+  const char *ar;
   float val;
 
-  if (!mrl || !mrl->prop || !mp || !ex)
-    return;
-
-  /* check if MRL actually has video stream */
-  if (!libvlc_media_player_has_vout (mp, ex))
+  if (!mrl || !mrl->prop || !mp)
     return;
 
   if (!mrl->prop->video)
-    mrl->prop->video = mrl_properties_video_new ();
+  {
+    int vid = 0;
+
+    /*
+     * HACK:
+     * VLC is not always able to found the streams with the following
+     * resources. But we can consider that they use always a video stream.
+     * Here we are sure that the video window is mapped in any cases.
+     */
+    switch (mrl->resource)
+    {
+    case MRL_RESOURCE_DVD:
+    case MRL_RESOURCE_DVDNAV:
+      vid = 1;
+      break;
+
+    default:
+      break;
+    }
+
+    if (vid || es)
+      mrl->prop->video = mrl_properties_video_new ();
+  }
+
+  if (!es)
+    return;
 
   video = mrl->prop->video;
 
-  video->width   = libvlc_video_get_width (mp, ex);
-  video->height  = libvlc_video_get_height (mp, ex);
-  video->aspect  = (uint32_t) (pl_atof (libvlc_video_get_aspect_ratio (mp, ex))
-                               * PLAYER_VIDEO_ASPECT_RATIO_MULT);
+  video->width   = es->u.video.i_width;
+  video->height  = es->u.video.i_height;
+
+  ar = libvlc_video_get_aspect_ratio (mp);
+  if (ar)
+    video->aspect = (uint32_t) (pl_atof (ar) * PLAYER_VIDEO_ASPECT_RATIO_MULT);
 
   video->streams = 0;
-  tracks = libvlc_video_get_track_description (mp, ex);
+  tracks = libvlc_video_get_track_description (mp);
   t = tracks;
   while (t)
   {
@@ -302,21 +356,20 @@ vlc_identify_video (mrl_t *mrl, libvlc_media_player_t *mp,
   }
   libvlc_track_description_release (tracks);
 
-  val = libvlc_media_player_get_fps (mp, ex);
+  val = libvlc_media_player_get_fps (mp);
   video->frameduration =
     (uint32_t) (val ? PLAYER_VIDEO_FRAMEDURATION_RATIO_DIV / val : 0);
 }
 
 static void
-vlc_identify_properties (mrl_t *mrl, libvlc_media_player_t *mp,
-                         libvlc_exception_t *ex)
+vlc_identify_properties (mrl_t *mrl, libvlc_media_player_t *mp)
 {
-  if (!mrl || !mrl->prop || !mp || !ex)
+  if (!mrl || !mrl->prop || !mp)
     return;
 
-  mrl->prop->seekable = libvlc_media_player_is_seekable (mp, ex);
+  mrl->prop->seekable = libvlc_media_player_is_seekable (mp);
 
-  mrl->prop->length = (uint32_t) libvlc_media_player_get_length (mp, ex);
+  mrl->prop->length = (uint32_t) libvlc_media_player_get_length (mp);
 }
 
 static void
@@ -326,14 +379,22 @@ vlc_identify (player_t *player, mrl_t *mrl, int flags)
   libvlc_media_t *media;
   char *uri = NULL;
   vlc_t *vlc;
-  const char *options[32] = { "--vout", "dummy", "--aout", "dummy" };
+  const char *options[] = {
+    ":vout=dummy",
+    ":aout=dummy",
+    ":sout=#description",
+  };
+  libvlc_media_track_info_t *esv = NULL, *esa = NULL;
+  libvlc_media_track_info_t *es = NULL;
   libvlc_state_t st = libvlc_NothingSpecial;
   int wait = 0;
+  unsigned int i;
+  unsigned int es_count;
 
   if (!player || !mrl)
     return;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->core || !vlc->mp)
     return;
 
@@ -341,47 +402,68 @@ vlc_identify (player_t *player, mrl_t *mrl, int flags)
   if (!uri)
     return;
 
-  mp = libvlc_media_player_new (vlc->core, &vlc->ex);
+  mp = libvlc_media_player_new (vlc->core);
   if (!mp)
     goto err_mp;
 
-  media = libvlc_media_new (vlc->core, uri, &vlc->ex);
+  media = libvlc_media_new_location (vlc->core, uri);
   if (!media)
     goto err_media;
 
-  libvlc_media_add_option (media, *options, &vlc->ex);
+  for (i = 0; i < ARRAY_NB_ELEMENTS (options); i++)
+    libvlc_media_add_option (media, options[i]);
 
-  libvlc_media_player_set_media (mp, media, &vlc->ex);
-  libvlc_media_player_play (mp, &vlc->ex);
+  libvlc_media_player_set_media (mp, media);
+  libvlc_media_player_play (mp);
 
+  /*
+   * FIXME
+   * Most of time, vlc_identify() is unable to retrieve the properties
+   * because libvlc must be playing in order to read the informations.
+   * The code below waits that the Playing state begins, but it is not
+   * sufficient for many streams. It suggests that at least one frame
+   * or more must be decoded.
+   */
   while (st <= libvlc_Buffering)
   {
-    st = libvlc_media_player_get_state (mp, &vlc->ex);
+    st = libvlc_media_player_get_state (mp);
     usleep (WAIT_PERIOD);
     wait += WAIT_PERIOD;
     if (wait >= WAIT_MAX)
       break;
   }
 
+  libvlc_media_parse (media);
+  es_count = libvlc_media_get_tracks_info (media, &es);
+  for (i = 0; i < es_count; i++)
+  {
+    if (!esv && es[i].i_type == libvlc_track_video)
+      esv = es + i;
+    else if (!esa && es[i].i_type == libvlc_track_audio)
+      esa = es + i;
+  }
+
   if (flags & IDENTIFY_VIDEO)
-    vlc_identify_video (mrl, mp, &vlc->ex);
+    vlc_identify_video (mrl, mp, esv);
 
   if (flags & IDENTIFY_AUDIO)
-    vlc_identify_audio (mrl, mp, &vlc->ex);
+    vlc_identify_audio (mrl, mp, esa);
 
   if (flags & IDENTIFY_METADATA)
-    vlc_identify_metadata (mrl, mp, &vlc->ex);
+    vlc_identify_metadata (mrl, mp);
 
   if (flags & IDENTIFY_PROPERTIES)
-    vlc_identify_properties (mrl, mp, &vlc->ex);
+    vlc_identify_properties (mrl, mp);
 
-  libvlc_media_player_stop (mp, &vlc->ex);
+  libvlc_media_player_stop (mp);
   libvlc_media_release (media);
 
+  PFREE (es);
+
  err_media:
   libvlc_media_player_release (mp);
  err_mp:
-  free (uri);
+  PFREE (uri);
 }
 
 /*****************************************************************************/
@@ -392,29 +474,31 @@ static init_status_t
 vlc_init (player_t *player)
 {
   vlc_t *vlc = NULL;
-  const char *vlc_argv[32] = { "vlc" };
+  const char *vlc_argv[32];
   libvlc_event_manager_t *ev;
-  int vlc_argc = 1;
+  int vlc_argc = 0;
+  int use_x11 = 0;
   unsigned int i;
+  uint32_t winid = 0;
 
   pl_log (player, PLAYER_MSG_INFO, MODULE_NAME, "init");
 
   if (!player)
     return PLAYER_INIT_ERROR;
 
-  //vlc_argv[vlc_argc++] = "-vv";
   vlc_argv[vlc_argc++] = "--no-stats";
-  vlc_argv[vlc_argc++] = "--intf";
-  vlc_argv[vlc_argc++] = "dummy";
   vlc_argv[vlc_argc++] = "--verbose";
   vlc_argv[vlc_argc++] = "0";
-  vlc_argv[vlc_argc++] = "--ignore-config";
-  vlc_argv[vlc_argc++] = "--reset-plugins-cache";
   vlc_argv[vlc_argc++] = "--no-media-library";
-  vlc_argv[vlc_argc++] = "--no-one-instance";
   vlc_argv[vlc_argc++] = "--no-osd";
   vlc_argv[vlc_argc++] = "--no-video-title-show" ;
 
+  if (player->x11_display)
+  {
+    vlc_argv[vlc_argc++] = "--x11-display";
+    vlc_argv[vlc_argc++] = player->x11_display;
+  }
+
   /* select the video output */
   switch (player->vo)
   {
@@ -426,18 +510,25 @@ vlc_init (player_t *player)
   case PLAYER_VO_X11:
     vlc_argv[vlc_argc++] = "--vout";
     vlc_argv[vlc_argc++] = "x11,dummy";
+    use_x11 = 1;
     break;
 
   case PLAYER_VO_XV:
     vlc_argv[vlc_argc++] = "--vout";
     vlc_argv[vlc_argc++] = "xvideo,dummy";
+    use_x11 = 1;
     break;
 
   case PLAYER_VO_GL:
     vlc_argv[vlc_argc++] = "--vout";
     vlc_argv[vlc_argc++] = "glx,dummy";
+    use_x11 = 1;
+    break;
+#endif /* USE_X11 */
+
+  case PLAYER_VO_AUTO:
+    use_x11 = 1;
     break;
-#endif
 
   default:
     return PLAYER_INIT_ERROR;
@@ -458,32 +549,62 @@ vlc_init (player_t *player)
   case PLAYER_AO_OSS:
     vlc_argv[vlc_argc++] = "--aout";
     vlc_argv[vlc_argc++] = "oss,dummy";
+
+  case PLAYER_AO_PULSE:
+    vlc_argv[vlc_argc++] = "--aout";
+    vlc_argv[vlc_argc++] = "pulse,dummy";
     break;
 
+  case PLAYER_AO_AUTO:
   default:
     break;
   }
 
-  vlc = (vlc_t *) player->priv;
-  libvlc_exception_init (&vlc->ex);
-  vlc->core = libvlc_new (vlc_argc, vlc_argv, &vlc->ex);
-  vlc_check_exception (player);
+  if (use_x11)
+  {
+#ifdef USE_X11
+    int rc;
+
+    rc = pl_x11_init (player);
+    if (!rc && player->vo != PLAYER_VO_AUTO)
+    {
+      pl_log (player, PLAYER_MSG_ERROR,
+              MODULE_NAME, "initialization for X has failed");
+      return PLAYER_INIT_ERROR;
+    }
+
+    winid = pl_x11_get_window (player->x11);
+#else /* USE_X11 */
+    pl_log (player, PLAYER_MSG_ERROR, MODULE_NAME,
+            "auto-detection for videoout is not enabled without X11 support");
+    return PLAYER_INIT_ERROR;
+#endif /* !USE_X11 */
+  }
+
+  vlc = player->priv;
+  vlc->core = libvlc_new (vlc_argc, vlc_argv);
 
   if (!vlc->core)
     return PLAYER_INIT_ERROR;
 
-  vlc->mp = libvlc_media_player_new (vlc->core, &vlc->ex);
+  vlc->mp = libvlc_media_player_new (vlc->core);
   if (!vlc->mp)
     return PLAYER_INIT_ERROR;
 
+  if (winid)
+    libvlc_media_player_set_xwindow (vlc->mp, winid);
+
+  libvlc_video_set_key_input   (vlc->mp, 0);
+  libvlc_video_set_mouse_input (vlc->mp, 0);
+
   /* register the event manager */
-  ev = libvlc_media_player_event_manager (vlc->mp, &vlc->ex);
+  ev = libvlc_media_player_event_manager (vlc->mp);
   if (!ev)
     return PLAYER_INIT_ERROR;
 
-  for (i = 0; i < (sizeof (mp_events) / sizeof (*mp_events)); i++)
+  for (i = 0; i < ARRAY_NB_ELEMENTS (mp_events); i++)
     libvlc_event_attach (ev, mp_events[i],
-                         vlc_event_callback, player, &vlc->ex);
+                         vlc_event_callback, player);
 
   return PLAYER_INIT_OK;
 }
@@ -492,24 +613,33 @@ static void
 vlc_uninit (player_t *player)
 {
   vlc_t *vlc = NULL;
+  libvlc_media_t *media;
 
   pl_log (player, PLAYER_MSG_INFO, MODULE_NAME, "uninit");
 
   if (!player)
     return;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
 
   if (!vlc)
     return;
 
-  libvlc_exception_clear (&vlc->ex);
+  if (vlc->mp)
+  {
+    media = libvlc_media_player_get_media (vlc->mp);
+    if (media)
+      libvlc_media_release (media);
+    libvlc_media_player_release (vlc->mp);
+  }
   if (vlc->core)
     libvlc_release (vlc->core);
-  if (vlc->mp)
-    libvlc_media_player_release (vlc->mp);
 
-  free (vlc);
+#ifdef USE_X11
+  pl_x11_uninit (player);
+#endif /* USE_X11 */
+
+  PFREE (vlc);
 }
 
 static void
@@ -523,7 +653,7 @@ vlc_set_verbosity (player_t *player, player_verbosity_level_t level)
   if (!player)
     return;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc)
     return;
 
@@ -545,7 +675,7 @@ vlc_set_verbosity (player_t *player, player_verbosity_level_t level)
   }
 
   if (vlc->core && verbosity != -1)
-    libvlc_set_log_verbosity (vlc->core, verbosity, &vlc->ex);
+    libvlc_set_log_verbosity (vlc->core, verbosity);
 }
 
 static void
@@ -592,23 +722,48 @@ vlc_mrl_video_snapshot (player_t *player, mrl_t *mrl, pl_unused int pos,
 {
   vlc_t *vlc;
   unsigned int width, height;
+  libvlc_media_track_info_t *es = NULL;
+  libvlc_media_track_info_t *esv = NULL;
+  libvlc_media_t *media;
+  unsigned int es_count;
+  unsigned int i;
 
   pl_log (player, PLAYER_MSG_VERBOSE, MODULE_NAME, "mrl_video_snapshot");
 
   if (!player || !mrl || !mrl->meta)
     return;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->mp)
     return;
 
-  if (!libvlc_media_player_has_vout (vlc->mp, &vlc->ex))
+  media = libvlc_media_player_get_media (vlc->mp);
+  if (!media)
     return;
 
-  width  = libvlc_video_get_width  (vlc->mp, &vlc->ex);
-  height = libvlc_video_get_height (vlc->mp, &vlc->ex);
+  /*
+   * FIXME
+   * Like vlc_identify and according to the documentation of libvlc, the media
+   * must be played at least one time, else ES array will be empty.
+   */
+  es_count = libvlc_media_get_tracks_info (media, &es);
+  for (i = 0; i < es_count; i++)
+    if (es[i].i_type == libvlc_track_video)
+    {
+      esv = es + i;
+      break;
+    }
 
-  libvlc_video_take_snapshot (vlc->mp, dst, width, height, &vlc->ex);
+  if (!esv)
+    goto out;
+
+  width  = esv->u.video.i_width;
+  height = esv->u.video.i_height;
+
+  libvlc_video_take_snapshot (vlc->mp, 0, dst, width, height);
+
+ out:
+  PFREE (es);
 }
 
 static int
@@ -622,11 +777,11 @@ vlc_get_time_pos (player_t *player)
   if (!player)
     return -1;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->mp)
     return -1;
 
-  time_pos = libvlc_media_player_get_time (vlc->mp, &vlc->ex);
+  time_pos = libvlc_media_player_get_time (vlc->mp);
 
   return (time_pos < 0.0) ? -1: (int) time_pos;
 }
@@ -642,11 +797,11 @@ vlc_get_percent_pos (player_t *player)
   if (!player)
     return -1;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->mp)
     return -1;
 
-  pos = libvlc_media_player_get_position (vlc->mp, &vlc->ex);
+  pos = libvlc_media_player_get_position (vlc->mp);
   return (pos < 0.0) ? -1 : (int) (pos * 100.0);
 }
 
@@ -663,7 +818,7 @@ vlc_playback_start (player_t *player)
   if (!player)
     return PLAYER_PB_FATAL;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->mp)
     return PLAYER_PB_ERROR;
 
@@ -679,14 +834,19 @@ vlc_playback_start (player_t *player)
     return PLAYER_PB_ERROR;
 
   pl_log (player, PLAYER_MSG_INFO, MODULE_NAME, "uri: %s", uri);
-  media = libvlc_media_new (vlc->core, uri, &vlc->ex);
-  free (uri);
+  media = libvlc_media_new_location (vlc->core, uri);
+  PFREE (uri);
 
   if (!media)
     return PLAYER_PB_ERROR;
 
-  libvlc_media_player_set_media (vlc->mp, media, &vlc->ex);
-  libvlc_media_player_play (vlc->mp, &vlc->ex);
+#ifdef USE_X11
+  if (MRL_USES_VO (mrl))
+    pl_x11_map (player);
+#endif /* USE_X11 */
+
+  libvlc_media_player_set_media (vlc->mp, media);
+  libvlc_media_player_play (vlc->mp);
 
   return PLAYER_PB_OK;
 }
@@ -694,6 +854,9 @@ vlc_playback_start (player_t *player)
 static void
 vlc_playback_stop (player_t *player)
 {
+#ifdef USE_X11
+  mrl_t *mrl;
+#endif /* USE_X11 */
   vlc_t *vlc;
   libvlc_media_t *media;
 
@@ -702,12 +865,18 @@ vlc_playback_stop (player_t *player)
   if (!player)
     return;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->mp)
     return;
 
-  media = libvlc_media_player_get_media (vlc->mp, &vlc->ex);
-  libvlc_media_player_stop (vlc->mp, &vlc->ex);
+#ifdef USE_X11
+  mrl = pl_playlist_get_mrl (player->playlist);
+  if (MRL_USES_VO (mrl))
+    pl_x11_unmap (player);
+#endif /* USE_X11 */
+
+  media = libvlc_media_player_get_media (vlc->mp);
+  libvlc_media_player_stop (vlc->mp);
   libvlc_media_release (media);
 }
 
@@ -721,15 +890,15 @@ vlc_playback_pause (player_t *player)
   if (!player)
     return PLAYER_PB_FATAL;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->mp)
     return PLAYER_PB_FATAL;
 
-  if (libvlc_media_player_is_playing (vlc->mp, &vlc->ex) &&
-      libvlc_media_player_can_pause (vlc->mp, &vlc->ex))
-    libvlc_media_player_pause (vlc->mp, &vlc->ex);
+  if (libvlc_media_player_is_playing (vlc->mp) &&
+      libvlc_media_player_can_pause (vlc->mp))
+    libvlc_media_player_pause (vlc->mp);
   else
-    libvlc_media_player_play (vlc->mp, &vlc->ex);
+    libvlc_media_player_play (vlc->mp);
 
   return PLAYER_PB_OK;
 }
@@ -745,7 +914,7 @@ vlc_playback_seek (player_t *player, int value, player_pb_seek_t seek)
   if (!player)
     return;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->mp)
     return;
 
@@ -756,23 +925,23 @@ vlc_playback_seek (player_t *player, int value, player_pb_seek_t seek)
   {
     libvlc_time_t pos_time, length;
 
-    length = libvlc_media_player_get_length (vlc->mp, &vlc->ex);
-    pos_time = libvlc_media_player_get_time (vlc->mp, &vlc->ex);
-    pos_time += value * 1000;
+    length = libvlc_media_player_get_length (vlc->mp);
+    pos_time = libvlc_media_player_get_time (vlc->mp);
+    pos_time += value;
 
     if (pos_time < 0)
       pos_time = 0;
     if (pos_time > length)
       break;
 
-    libvlc_media_player_set_time (vlc->mp, pos_time, &vlc->ex);
+    libvlc_media_player_set_time (vlc->mp, pos_time);
     break;
   }
   case PLAYER_PB_SEEK_PERCENT:
-    libvlc_media_player_set_position (vlc->mp, value, &vlc->ex);
+    libvlc_media_player_set_position (vlc->mp, value / 100.0);
     break;
   case PLAYER_PB_SEEK_ABSOLUTE:
-    libvlc_media_player_set_time (vlc->mp, (value * 1000), &vlc->ex);
+    libvlc_media_player_set_time (vlc->mp, value);
     break;
   }
 }
@@ -789,17 +958,17 @@ vlc_playback_seek_chapter (player_t *player, int value, int absolute)
   if (!player)
     return;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->mp)
     return;
 
   chapter = absolute ? value :
-    libvlc_media_player_get_chapter (vlc->mp, &vlc->ex) + value;
+    libvlc_media_player_get_chapter (vlc->mp) + value;
 
-  if (chapter > libvlc_media_player_get_chapter_count (vlc->mp, &vlc->ex))
+  if (chapter > libvlc_media_player_get_chapter_count (vlc->mp))
     return;
 
-  libvlc_media_player_set_chapter (vlc->mp, chapter, &vlc->ex);
+  libvlc_media_player_set_chapter (vlc->mp, chapter);
 }
 
 static void
@@ -813,11 +982,11 @@ vlc_playback_set_speed (player_t *player, float value)
   if (!player)
     return;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->mp)
     return;
 
-  libvlc_media_player_set_rate (vlc->mp, value, &vlc->ex);
+  libvlc_media_player_set_rate (vlc->mp, value);
 }
 
 static int
@@ -831,8 +1000,8 @@ vlc_audio_get_volume (player_t *player)
   if (!player)
     return volume;
 
-  vlc = (vlc_t *) player->priv;
-  volume = libvlc_audio_get_volume (vlc->core, &vlc->ex);
+  vlc = player->priv;
+  volume = libvlc_audio_get_volume (vlc->mp);
 
   return (volume < 0) ? -1 : volume;
 }
@@ -848,8 +1017,8 @@ vlc_audio_set_volume (player_t *player, int value)
   if (!player)
     return;
 
-  vlc = (vlc_t *) player->priv;
-  libvlc_audio_set_volume (vlc->core, value, &vlc->ex);
+  vlc = player->priv;
+  libvlc_audio_set_volume (vlc->mp, value);
 }
 
 static player_mute_t
@@ -863,8 +1032,8 @@ vlc_audio_get_mute (player_t *player)
   if (!player)
     return mute;
 
-  vlc = (vlc_t *) player->priv;
-  mute = libvlc_audio_get_mute (vlc->core, &vlc->ex) ?
+  vlc = player->priv;
+  mute = libvlc_audio_get_mute (vlc->mp) ?
     PLAYER_MUTE_ON : PLAYER_MUTE_OFF;
 
   return mute;
@@ -888,26 +1057,8 @@ vlc_audio_set_mute (player_t *player, player_mute_t value)
   if (!player)
     return;
 
-  vlc = (vlc_t *) player->priv;
-  libvlc_audio_set_mute (vlc->core, mute , &vlc->ex);
-}
-
-static void
-vlc_video_set_fullscreen (player_t *player, int value)
-{
-  vlc_t *vlc;
-
-  pl_log (player, PLAYER_MSG_VERBOSE,
-          MODULE_NAME, "video_set_fullscreen: %s", value ? "on" : "off");
-
-  if (!player)
-    return;
-
-  vlc = (vlc_t *) player->priv;
-  if (!vlc || !vlc->mp)
-    return;
-
-  libvlc_set_fullscreen (vlc->mp, value, &vlc->ex);
+  vlc = player->priv;
+  libvlc_audio_set_mute (vlc->mp, mute);
 }
 
 static void
@@ -922,7 +1073,7 @@ vlc_video_set_ar (player_t *player, float value)
   if (!player)
     return;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->mp)
     return;
 
@@ -941,7 +1092,7 @@ vlc_video_set_ar (player_t *player, float value)
   else
     return;
 
-  libvlc_video_set_aspect_ratio (vlc->mp, ar, &vlc->ex);
+  libvlc_video_set_aspect_ratio (vlc->mp, ar);
 }
 
 static void
@@ -955,17 +1106,17 @@ vlc_sub_select (player_t *player, int value)
   if (!player)
     return;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->mp)
     return;
 
-  max = libvlc_video_get_spu_count (vlc->mp, &vlc->ex);
+  max = libvlc_video_get_spu_count (vlc->mp);
   if (value < 0)
     value = 0;
   if (value > max)
     value = max;
 
-  libvlc_video_set_spu (vlc->mp, value, &vlc->ex);
+  libvlc_video_set_spu (vlc->mp, value);
 }
 
 static void
@@ -979,19 +1130,19 @@ vlc_sub_prev (player_t *player)
   if (!player)
     return;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->mp)
     return;
 
-  cur = libvlc_video_get_spu (vlc->mp, &vlc->ex);
-  max = libvlc_video_get_spu_count (vlc->mp, &vlc->ex);
+  cur = libvlc_video_get_spu (vlc->mp);
+  max = libvlc_video_get_spu_count (vlc->mp);
   val = cur - 1;
 
   /* if current is first one, just cycle */
   if (val < 0)
     val = max;
 
-  libvlc_video_set_spu (vlc->mp, val, &vlc->ex);
+  libvlc_video_set_spu (vlc->mp, val);
 }
 
 static void
@@ -1005,19 +1156,19 @@ vlc_sub_next (player_t *player)
   if (!player)
     return;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->mp)
     return;
 
-  cur = libvlc_video_get_spu (vlc->mp, &vlc->ex);
-  max = libvlc_video_get_spu_count (vlc->mp, &vlc->ex);
+  cur = libvlc_video_get_spu (vlc->mp);
+  max = libvlc_video_get_spu_count (vlc->mp);
   val = cur + 1;
 
   /* if current is last one, just cycle */
   if (val > max)
     val = 0;
 
-  libvlc_video_set_spu (vlc->mp, val, &vlc->ex);
+  libvlc_video_set_spu (vlc->mp, val);
 }
 
 static void
@@ -1030,17 +1181,17 @@ vlc_dvd_title_set (player_t *player, int value)
   if (!player)
     return;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->mp)
     return;
 
   if (value < 1 || value > 99)
     return;
 
-  if (value > libvlc_media_player_get_title_count (vlc->mp, &vlc->ex))
+  if (value > libvlc_media_player_get_title_count (vlc->mp))
     return;
 
-  libvlc_media_player_set_title (vlc->mp, value, &vlc->ex);
+  libvlc_media_player_set_title (vlc->mp, value);
 }
 
 static void
@@ -1054,11 +1205,11 @@ vlc_dvd_title_prev (player_t *player)
   if (!player)
     return;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->mp)
     return;
 
-  value = libvlc_media_player_get_title_count (vlc->mp, &vlc->ex) - 1;
+  value = libvlc_media_player_get_title_count (vlc->mp) - 1;
   vlc_dvd_title_set (player, value);
 }
 
@@ -1073,11 +1224,11 @@ vlc_dvd_title_next (player_t *player)
   if (!player)
     return;
 
-  vlc = (vlc_t *) player->priv;
+  vlc = player->priv;
   if (!vlc || !vlc->mp)
     return;
 
-  value = libvlc_media_player_get_title_count (vlc->mp, &vlc->ex) + 1;
+  value = libvlc_media_player_get_title_count (vlc->mp) + 1;
   vlc_dvd_title_set (player, value);
 }
 
@@ -1090,6 +1241,8 @@ pl_supported_resources_vlc (mrl_resource_t res)
 {
   switch (res)
   {
+  case MRL_RESOURCE_DVD:
+  case MRL_RESOURCE_DVDNAV:
   case MRL_RESOURCE_FILE:
   case MRL_RESOURCE_FTP:
   case MRL_RESOURCE_HTTP:
@@ -1111,7 +1264,7 @@ pl_register_functions_vlc (void)
 {
   player_funcs_t *funcs = NULL;
 
-  funcs = calloc (1, sizeof (player_funcs_t));
+  funcs = PCALLOC (player_funcs_t, 1);
   if (!funcs)
     return NULL;
 
@@ -1128,6 +1281,7 @@ pl_register_functions_vlc (void)
   funcs->set_framedrop      = NULL;
   funcs->set_mouse_pos      = NULL;
   funcs->osd_show_text      = NULL;
+  funcs->osd_state          = NULL;
 
   funcs->pb_start           = vlc_playback_start;
   funcs->pb_stop            = vlc_playback_stop;
@@ -1145,7 +1299,6 @@ pl_register_functions_vlc (void)
   funcs->audio_prev         = NULL;
   funcs->audio_next         = NULL;
 
-  funcs->video_set_fs       = vlc_video_set_fullscreen;
   funcs->video_set_aspect   = NULL;
   funcs->video_set_panscan  = NULL;
   funcs->video_set_ar       = vlc_video_set_ar;
@@ -1175,7 +1328,7 @@ pl_register_functions_vlc (void)
   funcs->radio_channel_prev = NULL;
   funcs->radio_channel_next = NULL;
 
-  funcs->vdr                = NULL;
+  funcs->vdr                = PL_NOT_SUPPORTED;
 
   return funcs;
 }
@@ -1185,7 +1338,7 @@ pl_register_private_vlc (void)
 {
   vlc_t *vlc = NULL;
 
-  vlc = calloc (1, sizeof (vlc_t));
+  vlc = PCALLOC (vlc_t, 1);
   if (!vlc)
     return NULL;
 
