@@ -62,7 +62,7 @@ xine_player_event_listener_cb (void *user_data, const xine_event_t *event)
 {
   player_t *player = NULL;
 
-  player = (player_t *) user_data;
+  player = user_data;
   if (!player)
     return;
 
@@ -72,11 +72,10 @@ xine_player_event_listener_cb (void *user_data, const xine_event_t *event)
   {
     pl_log (player, PLAYER_MSG_INFO,
             MODULE_NAME, "Playback of stream has ended");
-    player_event_send (player, PLAYER_EVENT_PLAYBACK_FINISHED, NULL);
+    player_event_send (player, PLAYER_EVENT_PLAYBACK_FINISHED);
 
 #ifdef USE_X11
-    if (player->x11)
-      pl_x11_unmap (player);
+    pl_x11_unmap (player);
 #endif /* USE_X11 */
     break;
   }
@@ -99,7 +98,7 @@ send_event (player_t *player, int event, void *data, int data_size)
   if (!player)
     return;
 
-  x = (xine_player_t *) player->priv;
+  x = player->priv;
 
   if (!x || !x->stream)
     return;
@@ -112,6 +111,110 @@ send_event (player_t *player, int event, void *data, int data_size)
   xine_event_send (x->stream, &xine_event);
 }
 
+static char *
+xi_resource_get_uri_local (const char *protocol,
+                           mrl_resource_local_args_t *args)
+{
+  if (!args || !args->location || !protocol)
+    return NULL;
+
+  if (strchr (args->location, ':')
+      && strncmp (args->location, protocol, strlen (protocol)))
+    return NULL;
+
+  return strdup (args->location);
+}
+
+static char *
+xi_resource_get_uri_dvd (const char *protocol,
+                         mrl_resource_videodisc_args_t *args)
+{
+  char *uri;
+  char title_start[8] = "";
+  size_t size;
+
+  if (!args || !protocol)
+    return NULL;
+
+  size = strlen (protocol);
+
+  if (args->device)
+    size += strlen (args->device);
+
+  if (args->title_start)
+  {
+    size += 1 + pl_count_nb_dec (args->title_start);
+    snprintf (title_start, sizeof (title_start), "/%i", args->title_start);
+  }
+
+  size++;
+  uri = malloc (size);
+  if (uri)
+    snprintf (uri, size, "%s%s%s",
+              protocol, args->device ? args->device : "", title_start);
+
+  return uri;
+}
+
+static char *
+xi_resource_get_uri_vdr (const char *protocol, mrl_resource_tv_args_t *args)
+{
+  char *uri;
+  size_t size;
+
+  if (!protocol)
+    return NULL;
+
+  size = strlen (protocol);
+
+  if (!args || !args->device)
+    return strdup (protocol);
+
+  if (args->driver)
+    size += 1 + strlen (args->driver);
+
+  size += strlen (args->device) + 1;
+  uri = malloc (size);
+  snprintf (uri, size, "%s%s%s%s",
+            protocol, args->device,
+            args->driver ? "#" : "",
+            args->driver ? args->driver : "");
+
+  return uri;
+}
+
+static char *
+xi_resource_get_uri_network (const char *protocol,
+                             mrl_resource_network_args_t *args)
+{
+  char *uri, *host_file;
+  size_t size;
+
+  if (!args || !args->url || !protocol)
+    return NULL;
+
+  size = strlen (protocol);
+
+  if (strstr (args->url, protocol) == args->url)
+    host_file = strdup (args->url + size);
+  else
+    host_file = strdup (args->url);
+
+  if (!host_file)
+    return NULL;
+
+  size += strlen (host_file);
+
+  size++;
+  uri = malloc (size);
+  if (uri)
+    snprintf (uri, size, "%s%s", protocol, host_file);
+
+  PFREE (host_file);
+
+  return uri;
+}
+
 static char *
 xine_resource_get_uri (mrl_t *mrl)
 {
@@ -127,7 +230,12 @@ xine_resource_get_uri (mrl_t *mrl)
     [MRL_RESOURCE_VDR]      = "vdr:/",
 
     /* Network Streams */
+    [MRL_RESOURCE_HTTP]     = "http://",
+    [MRL_RESOURCE_MMS]      = "mms://",
     [MRL_RESOURCE_NETVDR]   = "netvdr://",
+    [MRL_RESOURCE_RTP]      = "rtp://",
+    [MRL_RESOURCE_TCP]      = "tcp://",
+    [MRL_RESOURCE_UDP]      = "udp://",
 
     [MRL_RESOURCE_UNKNOWN]  = NULL
   };
@@ -138,91 +246,22 @@ xine_resource_get_uri (mrl_t *mrl)
   switch (mrl->resource)
   {
   case MRL_RESOURCE_FILE: /* file:location */
-  {
-    const char *protocol = protocols[mrl->resource];
-    mrl_resource_local_args_t *args = mrl->priv;
-
-    if (!args || !args->location)
-      return NULL;
-
-    if (strchr (args->location, ':')
-        && strncmp (args->location, protocol, strlen (protocol)))
-    {
-      return NULL;
-    }
-
-    return strdup (args->location);
-  }
+    return xi_resource_get_uri_local (protocols[mrl->resource], mrl->priv);
 
   case MRL_RESOURCE_DVD:    /* dvd:device/title_start */
   case MRL_RESOURCE_DVDNAV: /* dvd:device/title_start */
-  {
-    char *uri;
-    const char *protocol = protocols[mrl->resource];
-    char title_start[8] = "";
-    size_t size = strlen (protocol);
-    mrl_resource_videodisc_args_t *args;
-
-    args = mrl->priv;
-    if (!args)
-      break;
-
-    if (args->device)
-      size += strlen (args->device);
-
-    if (args->title_start)
-    {
-      size += 1 + pl_count_nb_dec (args->title_start);
-      snprintf (title_start, sizeof (title_start), "/%i", args->title_start);
-    }
-
-    size++;
-    uri = malloc (size);
-    if (!uri)
-      break;
-
-    snprintf (uri, size, "%s%s%s",
-              protocol, args->device ? args->device : "", title_start);
-
-    return uri;
-  }
+    return xi_resource_get_uri_dvd (protocols[mrl->resource], mrl->priv);
 
   case MRL_RESOURCE_VDR: /* vdr:/device#driver */
-  {
-    char *uri;
-    const char *protocol = protocols[mrl->resource];
-    size_t size = strlen (protocol);
-    mrl_resource_tv_args_t *args = mrl->priv;
-
-    if (!args || !args->device)
-      return strdup (protocol);
-
-    if (args->driver)
-      size += 1 + strlen (args->driver);
-
-    size += strlen (args->device) + 1;
-    uri = malloc (size);
-    snprintf (uri, size, "%s%s%s%s",
-              protocol, args->device,
-              args->driver ? "#" : "",
-              args->driver ? args->driver : "");
-
-    return uri;
-  }
+    return xi_resource_get_uri_vdr (protocols[mrl->resource], mrl->priv);
 
+  case MRL_RESOURCE_HTTP:   /* http://host...     */
+  case MRL_RESOURCE_MMS:    /* mms://host...      */
   case MRL_RESOURCE_NETVDR: /* netvdr://host:port */
-  {
-    const char *protocol = protocols[mrl->resource];
-    mrl_resource_network_args_t *args = mrl->priv;
-
-    if (!args || !args->url)
-      return NULL;
-
-    if (strncmp (args->url, protocol, strlen (protocol)))
-      return NULL;
-
-    return strdup (args->url);
-  }
+  case MRL_RESOURCE_RTP:    /* rtp://host:port    */
+  case MRL_RESOURCE_TCP:    /* tcp://host:port    */
+  case MRL_RESOURCE_UDP:    /* udp://host:port    */
+    return xi_resource_get_uri_network (protocols[mrl->resource], mrl->priv);
 
   default:
     break;
@@ -247,8 +286,7 @@ xine_identify_metadata_dvd (mrl_t *mrl, xine_stream_t *stream)
   s = xine_get_meta_info (stream, XINE_META_INFO_TITLE);
   if (s)
   {
-    if (dvd->volumeid)
-      free (dvd->volumeid);
+    PFREE (dvd->volumeid);
     dvd->volumeid = strdup (s);
   }
 
@@ -265,56 +303,49 @@ xine_identify_metadata_clip (mrl_t *mrl, xine_stream_t *stream)
   s = xine_get_meta_info (stream, XINE_META_INFO_TITLE);
   if (s)
   {
-    if (meta->title)
-      free (meta->title);
+    PFREE (meta->title);
     meta->title = strdup (s);
   }
 
   s = xine_get_meta_info (stream, XINE_META_INFO_ARTIST);
   if (s)
   {
-    if (meta->artist)
-      free (meta->artist);
+    PFREE (meta->artist);
     meta->artist = strdup (s);
   }
 
   s = xine_get_meta_info (stream, XINE_META_INFO_GENRE);
   if (s)
   {
-    if (meta->genre)
-      free (meta->genre);
+    PFREE (meta->genre);
     meta->genre = strdup (s);
   }
 
   s = xine_get_meta_info (stream, XINE_META_INFO_ALBUM);
   if (s)
   {
-    if (meta->album)
-      free (meta->album);
+    PFREE (meta->album);
     meta->album = strdup (s);
   }
 
   s = xine_get_meta_info (stream, XINE_META_INFO_YEAR);
   if (s)
   {
-    if (meta->year)
-      free (meta->year);
+    PFREE (meta->year);
     meta->year = strdup (s);
   }
 
   s = xine_get_meta_info (stream, XINE_META_INFO_TRACK_NUMBER);
   if (s)
   {
-    if (meta->track)
-      free (meta->track);
+    PFREE (meta->track);
     meta->track = strdup (s);
   }
 
   s = xine_get_meta_info (stream, XINE_META_INFO_COMMENT);
   if (s)
   {
-    if (meta->comment)
-      free (meta->comment);
+    PFREE (meta->comment);
     meta->comment = strdup (s);
   }
 }
@@ -358,8 +389,7 @@ xine_identify_audio (mrl_t *mrl, xine_stream_t *stream)
   s = xine_get_meta_info (stream, XINE_META_INFO_AUDIOCODEC);
   if (s)
   {
-    if (audio->codec)
-      free (audio->codec);
+    PFREE (audio->codec);
     audio->codec = strdup (s);
   }
 
@@ -396,8 +426,7 @@ xine_identify_video (mrl_t *mrl, xine_stream_t *stream)
   s = xine_get_meta_info (stream, XINE_META_INFO_VIDEOCODEC);
   if (s)
   {
-    if (video->codec)
-      free (video->codec);
+    PFREE (video->codec);
     video->codec = strdup (s);
   }
 
@@ -452,7 +481,7 @@ xine_identify (player_t *player, mrl_t *mrl, int flags)
   if (!player || !mrl)
     return;
 
-  x = (xine_player_t *) player->priv;
+  x = player->priv;
 
   if (!x)
     return;
@@ -497,7 +526,7 @@ xine_identify (player_t *player, mrl_t *mrl, int flags)
  err_vo:
   xine_close_audio_driver (x->xine, ao);
  err_ao:
-  free (uri);
+  PFREE (uri);
 }
 
 /*****************************************************************************/
@@ -521,8 +550,7 @@ xine_player_init (player_t *player)
   if (!player)
     return PLAYER_INIT_ERROR;
 
-  x = (xine_player_t *) player->priv;
-
+  x = player->priv;
   if (!x)
     return PLAYER_INIT_ERROR;
 
@@ -534,7 +562,7 @@ xine_player_init (player_t *player)
     char *cfgfile = malloc (cfgfile_len);
     snprintf (cfgfile, cfgfile_len, "%s/.xine/config", homedir);
     xine_config_load (x->xine, cfgfile);
-    free (cfgfile);
+    PFREE (cfgfile);
   }
   xine_init (x->xine);
   xine_engine_set_param (x->xine,
@@ -562,10 +590,17 @@ xine_player_init (player_t *player)
     id_vo = "xv";
     break;
 
+#ifdef USE_XLIB_HACK
   case PLAYER_VO_GL:
     use_x11 = 1;
     id_vo = "opengl";
     break;
+
+  case PLAYER_VO_VDPAU:
+    use_x11 = 1;
+    id_vo = "vdpau";
+    break;
+#endif /* USE_XLIB_HACK */
 #endif /* USE_X11 */
 
   case PLAYER_VO_FB:
@@ -573,6 +608,11 @@ xine_player_init (player_t *player)
     visual = XINE_VISUAL_TYPE_FB;
     break;
 
+  case PLAYER_VO_DIRECTFB:
+    id_vo = "directfb";
+    visual = XINE_VISUAL_TYPE_FB;
+    break;
+
   case PLAYER_VO_AUTO:
     use_x11 = 1;
     break;
@@ -601,17 +641,22 @@ xine_player_init (player_t *player)
     else
     {
       data = pl_x11_get_data (player->x11);
-      visual = XINE_VISUAL_TYPE_X11_2;
+#ifdef USE_XLIB_HACK
+      visual = XINE_VISUAL_TYPE_X11;
+#else /* USE_XLIB_HACK */
+      visual = XINE_VISUAL_TYPE_XCB;
+#endif /* !USE_XLIB_HACK */
     }
-#else
+#else /* USE_X11 */
     pl_log (player, PLAYER_MSG_ERROR, MODULE_NAME,
             "auto-detection for videoout is not enabled without X11 support");
     return PLAYER_INIT_ERROR;
-#endif /* USE_X11 */
+#endif /* !USE_X11 */
   }
 
   /* init video output driver */
-  if (!(x->vo_port = xine_open_video_driver (x->xine, id_vo, visual, data)))
+  x->vo_port = xine_open_video_driver (x->xine, id_vo, visual, data);
+  if (!x->vo_port)
   {
     pl_log (player, PLAYER_MSG_ERROR,
             MODULE_NAME, "xine can't init '%s' video driver",
@@ -634,6 +679,10 @@ xine_player_init (player_t *player)
     id_ao = "oss";
     break;
 
+  case PLAYER_AO_PULSE:
+    id_ao = "pulseaudio";
+    break;
+
   case PLAYER_AO_AUTO:
     break;
 
@@ -644,7 +693,8 @@ xine_player_init (player_t *player)
   }
 
   /* init audio output driver */
-  if (!(x->ao_port = xine_open_audio_driver (x->xine, id_ao, NULL)))
+  x->ao_port = xine_open_audio_driver (x->xine, id_ao, NULL);
+  if (!x->ao_port)
   {
     pl_log (player, PLAYER_MSG_ERROR,
             MODULE_NAME, "xine can't init '%s' audio driver",
@@ -660,13 +710,9 @@ xine_player_init (player_t *player)
                                      xine_player_event_listener_cb, player);
 
 #ifdef USE_X11
-  if (player->x11 && pl_x11_get_display (player->x11))
-  {
-    xine_port_send_gui_data (x->vo_port, XINE_GUI_SEND_DRAWABLE_CHANGED,
-                             (void *) pl_x11_get_window (player->x11));
-    xine_port_send_gui_data (x->vo_port, XINE_GUI_SEND_VIDEOWIN_VISIBLE,
-                             (void *) 1);
-  }
+  if (player->x11)
+    xine_port_send_gui_data (x->vo_port,
+                             XINE_GUI_SEND_VIDEOWIN_VISIBLE, (void *) 1);
 #endif /* USE_X11 */
 
   return PLAYER_INIT_OK;
@@ -682,8 +728,7 @@ xine_player_uninit (player_t *player)
   if (!player)
     return;
 
-  x = (xine_player_t *) player->priv;
-
+  x = player->priv;
   if (!x)
     return;
 
@@ -705,11 +750,10 @@ xine_player_uninit (player_t *player)
     xine_exit (x->xine);
 
 #ifdef USE_X11
-  if (player->x11)
-    pl_x11_uninit (player);
+  pl_x11_uninit (player);
 #endif /* USE_X11 */
 
-  free (x);
+  PFREE (x);
 }
 
 static void
@@ -721,7 +765,7 @@ xine_player_set_verbosity (player_t *player, player_verbosity_level_t level)
   if (!player)
     return;
 
-  x = (xine_player_t *) player->priv;
+  x = player->priv;
   if (!x)
     return;
 
@@ -797,9 +841,8 @@ xine_player_get_time_pos (player_t *player)
   if (!player)
     return -1;
 
-  x = (xine_player_t *) player->priv;
-
-  if (!x || !x->stream)
+  x = player->priv;
+  if (!x->stream)
     return -1;
 
   ret = xine_get_pos_length (x->stream, NULL, &time_pos, NULL);
@@ -889,7 +932,7 @@ xine_player_playback_start (player_t *player)
   if (!player)
     return PLAYER_PB_FATAL;
 
-  x = (xine_player_t *) player->priv;
+  x = player->priv;
 
   mrl_c = pl_playlist_get_mrl (player->playlist);
   if (!x->stream || !mrl_c)
@@ -914,20 +957,20 @@ xine_player_playback_start (player_t *player)
   else
     mrl = strdup (uri);
 
-  free (uri);
+  PFREE (uri);
 
   if (!mrl)
     return PLAYER_PB_ERROR;
 
 #ifdef USE_X11
-  if (player->x11 && !mrl_uses_vo (mrl_c))
+  if (MRL_USES_VO (mrl_c))
     pl_x11_map (player);
 #endif /* USE_X11 */
 
   xine_open (x->stream, mrl);
   xine_play (x->stream, 0, 0);
 
-  free (mrl);
+  PFREE (mrl);
 
   return PLAYER_PB_OK;
 }
@@ -945,14 +988,14 @@ xine_player_playback_stop (player_t *player)
   if (!player)
     return;
 
-  x = (xine_player_t *) player->priv;
+  x = player->priv;
 
   if (!x->stream)
     return;
 
 #ifdef USE_X11
   mrl = pl_playlist_get_mrl (player->playlist);
-  if (player->x11 && !mrl_uses_vo (mrl))
+  if (MRL_USES_VO (mrl))
     pl_x11_unmap (player);
 #endif /* USE_X11 */
 
@@ -970,7 +1013,7 @@ xine_player_playback_pause (player_t *player)
   if (!player)
     return PLAYER_PB_FATAL;
 
-  x = (xine_player_t *) player->priv;
+  x = player->priv;
 
   if (!x->stream)
     return PLAYER_PB_ERROR;
@@ -995,7 +1038,7 @@ xine_player_playback_seek (player_t *player, int value, player_pb_seek_t seek)
   if (!player)
     return;
 
-  x = (xine_player_t *) player->priv;
+  x = player->priv;
 
   if (!x->stream)
     return;
@@ -1006,14 +1049,14 @@ xine_player_playback_seek (player_t *player, int value, player_pb_seek_t seek)
   {
   default:
   case PLAYER_PB_SEEK_RELATIVE:
-    pos_time += value * 1000;
+    pos_time += value;
     break;
   case PLAYER_PB_SEEK_PERCENT:
     pos_percent = (1 << 16) * value / 100;
     pos_time = 0;
     break;
   case PLAYER_PB_SEEK_ABSOLUTE:
-    pos_time = value * 1000;
+    pos_time = value;
     break;
   }
 
@@ -1025,6 +1068,27 @@ xine_player_playback_seek (player_t *player, int value, player_pb_seek_t seek)
   xine_play (x->stream, pos_percent, pos_time);
 }
 
+static void
+xine_player_playback_set_speed (player_t *player, float value)
+{
+  int speed;
+  xine_player_t *x = NULL;
+
+  pl_log (player, PLAYER_MSG_VERBOSE,
+          MODULE_NAME, "playback_set_speed: %f", value);
+
+  if (!player)
+    return;
+
+  x = player->priv;
+
+  if (!x->stream)
+    return;
+
+  speed = (int) (value * XINE_FINE_SPEED_NORMAL);
+  xine_set_param (x->stream, XINE_PARAM_FINE_SPEED, speed);
+}
+
 static int
 xine_player_audio_get_volume (player_t *player)
 {
@@ -1035,7 +1099,7 @@ xine_player_audio_get_volume (player_t *player)
   if (!player)
     return -1;
 
-  x = (xine_player_t *) player->priv;
+  x = player->priv;
 
   if (!x->stream)
     return -1;
@@ -1054,7 +1118,7 @@ xine_player_audio_set_volume (player_t *player, int value)
   if (!player)
     return;
 
-  x = (xine_player_t *) player->priv;
+  x = player->priv;
 
   if (!x->stream)
     return;
@@ -1072,7 +1136,7 @@ xine_player_audio_get_mute (player_t *player)
   if (!player)
     return PLAYER_MUTE_UNKNOWN;
 
-  x = (xine_player_t *) player->priv;
+  x = player->priv;
 
   if (!x->stream)
     return PLAYER_MUTE_UNKNOWN;
@@ -1101,7 +1165,7 @@ xine_player_audio_set_mute (player_t *player, player_mute_t value)
   if (!player)
     return;
 
-  x = (xine_player_t *) player->priv;
+  x = player->priv;
 
   if (!x->stream)
     return;
@@ -1109,6 +1173,52 @@ xine_player_audio_set_mute (player_t *player, player_mute_t value)
   xine_set_param (x->stream, XINE_PARAM_AUDIO_MUTE, mute);
 }
 
+static void
+xine_player_video_set_ar (player_t *player, float value)
+{
+  xine_player_t *x;
+  mrl_t *mrl;
+  int ar;
+
+  pl_log (player, PLAYER_MSG_VERBOSE, MODULE_NAME, "video_set_ar: %.2f", value);
+
+  if (!player)
+    return;
+
+  x = player->priv;
+
+  if (!x->stream)
+    return;
+
+  mrl = pl_playlist_get_mrl (player->playlist);
+  if (!MRL_USES_VO (mrl))
+    return;
+
+  /* use original aspect ratio if value is 0.0 */
+  if (!value)
+  {
+    mrl_properties_video_t *video = mrl->prop->video;
+    player->aspect = video->aspect / PLAYER_VIDEO_ASPECT_RATIO_MULT;
+    ar = XINE_VO_ASPECT_AUTO;
+  }
+  else
+  {
+    player->aspect = value;
+    if (value >= 2.11)
+      ar = XINE_VO_ASPECT_DVB;
+    else if (value >= 16.0 / 9.0)
+      ar = XINE_VO_ASPECT_ANAMORPHIC;
+    else if (value >= 4.0 / 3.0)
+      ar = XINE_VO_ASPECT_4_3;
+    else if (value >= 1.0)
+      ar = XINE_VO_ASPECT_SQUARE;
+    else
+      return;
+  }
+
+  xine_set_param (x->stream, XINE_PARAM_VO_ASPECT_RATIO, ar);
+}
+
 static void
 xine_player_sub_set_delay (player_t *player, int value)
 {
@@ -1123,7 +1233,7 @@ xine_player_sub_set_delay (player_t *player, int value)
   if (!player)
     return;
 
-  x = (xine_player_t *) player->priv;
+  x = player->priv;
 
   if (!x->stream)
     return;
@@ -1174,7 +1284,7 @@ xine_player_dvd_nav (player_t *player, player_dvdnav_t value)
 
   case PLAYER_DVDNAV_MOUSECLICK:
     event = XINE_EVENT_INPUT_MOUSE_BUTTON;
-    input = calloc (1, sizeof (xine_input_data_t));
+    input = PCALLOC (xine_input_data_t, 1);
     if (!input)
       break;
 
@@ -1189,243 +1299,78 @@ xine_player_dvd_nav (player_t *player, player_dvdnav_t value)
 
   send_event (player, event, input, input ? sizeof (*input) : 0);
 
-  if (input)
-    free (input);
+  PFREE (input);
 }
 
 static void
 xine_player_vdr (player_t *player, player_vdr_t value)
 {
-  int event;
+  static const player_vdr_t events[] = {
+    [PLAYER_VDR_UP]               = XINE_EVENT_INPUT_UP,
+    [PLAYER_VDR_DOWN]             = XINE_EVENT_INPUT_DOWN,
+    [PLAYER_VDR_LEFT]             = XINE_EVENT_INPUT_LEFT,
+    [PLAYER_VDR_RIGHT]            = XINE_EVENT_INPUT_RIGHT,
+    [PLAYER_VDR_OK]               = XINE_EVENT_INPUT_SELECT,
+    [PLAYER_VDR_BACK]             = XINE_EVENT_VDR_BACK,
+    [PLAYER_VDR_CHANNELPLUS]      = XINE_EVENT_VDR_CHANNELPLUS,
+    [PLAYER_VDR_CHANNELMINUS]     = XINE_EVENT_VDR_CHANNELMINUS,
+    [PLAYER_VDR_RED]              = XINE_EVENT_VDR_RED,
+    [PLAYER_VDR_GREEN]            = XINE_EVENT_VDR_GREEN,
+    [PLAYER_VDR_YELLOW]           = XINE_EVENT_VDR_YELLOW,
+    [PLAYER_VDR_BLUE]             = XINE_EVENT_VDR_BLUE,
+    [PLAYER_VDR_PLAY]             = XINE_EVENT_VDR_PLAY,
+    [PLAYER_VDR_PAUSE]            = XINE_EVENT_VDR_PAUSE,
+    [PLAYER_VDR_STOP]             = XINE_EVENT_VDR_STOP,
+    [PLAYER_VDR_RECORD]           = XINE_EVENT_VDR_RECORD,
+    [PLAYER_VDR_FASTFWD]          = XINE_EVENT_VDR_FASTFWD,
+    [PLAYER_VDR_FASTREW]          = XINE_EVENT_VDR_FASTREW,
+    [PLAYER_VDR_POWER]            = XINE_EVENT_VDR_POWER,
+    [PLAYER_VDR_SCHEDULE]         = XINE_EVENT_VDR_SCHEDULE,
+    [PLAYER_VDR_CHANNELS]         = XINE_EVENT_VDR_CHANNELS,
+    [PLAYER_VDR_TIMERS]           = XINE_EVENT_VDR_TIMERS,
+    [PLAYER_VDR_RECORDINGS]       = XINE_EVENT_VDR_RECORDINGS,
+    [PLAYER_VDR_MENU]             = XINE_EVENT_INPUT_MENU1,
+    [PLAYER_VDR_SETUP]            = XINE_EVENT_VDR_SETUP,
+    [PLAYER_VDR_COMMANDS]         = XINE_EVENT_VDR_COMMANDS,
+    [PLAYER_VDR_0]                = XINE_EVENT_INPUT_NUMBER_0,
+    [PLAYER_VDR_1]                = XINE_EVENT_INPUT_NUMBER_1,
+    [PLAYER_VDR_2]                = XINE_EVENT_INPUT_NUMBER_2,
+    [PLAYER_VDR_3]                = XINE_EVENT_INPUT_NUMBER_3,
+    [PLAYER_VDR_4]                = XINE_EVENT_INPUT_NUMBER_4,
+    [PLAYER_VDR_5]                = XINE_EVENT_INPUT_NUMBER_5,
+    [PLAYER_VDR_6]                = XINE_EVENT_INPUT_NUMBER_6,
+    [PLAYER_VDR_7]                = XINE_EVENT_INPUT_NUMBER_7,
+    [PLAYER_VDR_8]                = XINE_EVENT_INPUT_NUMBER_8,
+    [PLAYER_VDR_9]                = XINE_EVENT_INPUT_NUMBER_9,
+    [PLAYER_VDR_USER_1]           = XINE_EVENT_VDR_USER1,
+    [PLAYER_VDR_USER_2]           = XINE_EVENT_VDR_USER2,
+    [PLAYER_VDR_USER_3]           = XINE_EVENT_VDR_USER3,
+    [PLAYER_VDR_USER_4]           = XINE_EVENT_VDR_USER4,
+    [PLAYER_VDR_USER_5]           = XINE_EVENT_VDR_USER5,
+    [PLAYER_VDR_USER_6]           = XINE_EVENT_VDR_USER6,
+    [PLAYER_VDR_USER_7]           = XINE_EVENT_VDR_USER7,
+    [PLAYER_VDR_USER_8]           = XINE_EVENT_VDR_USER8,
+    [PLAYER_VDR_USER_9]           = XINE_EVENT_VDR_USER9,
+    [PLAYER_VDR_VOLPLUS]          = XINE_EVENT_VDR_VOLPLUS,
+    [PLAYER_VDR_VOLMINUS]         = XINE_EVENT_VDR_VOLMINUS,
+    [PLAYER_VDR_MUTE]             = XINE_EVENT_VDR_MUTE,
+    [PLAYER_VDR_AUDIO]            = XINE_EVENT_VDR_AUDIO,
+    [PLAYER_VDR_INFO]             = XINE_EVENT_VDR_INFO,
+    [PLAYER_VDR_CHANNELPREVIOUS]  = XINE_EVENT_VDR_CHANNELPREVIOUS,
+    [PLAYER_VDR_NEXT]             = XINE_EVENT_INPUT_NEXT,
+    [PLAYER_VDR_PREVIOUS]         = XINE_EVENT_INPUT_PREVIOUS,
+    [PLAYER_VDR_SUBTITLES]        = XINE_EVENT_VDR_SUBTITLES,
+  };
 
-  pl_log (player, PLAYER_MSG_VERBOSE, MODULE_NAME, "vdr: %i", value);
+  pl_log (player, PLAYER_MSG_VERBOSE, MODULE_NAME, "vdr: %u", value);
 
   if (!player)
     return;
 
-  switch (value)
-  {
-  case PLAYER_VDR_UP:
-    event = XINE_EVENT_INPUT_UP;
-    break;
-
-  case PLAYER_VDR_DOWN:
-    event = XINE_EVENT_INPUT_DOWN;
-    break;
-
-  case PLAYER_VDR_LEFT:
-    event = XINE_EVENT_INPUT_LEFT;
-    break;
-
-  case PLAYER_VDR_RIGHT:
-    event = XINE_EVENT_INPUT_RIGHT;
-    break;
-
-  case PLAYER_VDR_OK:
-    event = XINE_EVENT_INPUT_SELECT;
-    break;
-
-  case PLAYER_VDR_BACK:
-    event = XINE_EVENT_VDR_BACK;
-    break;
-
-  case PLAYER_VDR_CHANNELPLUS:
-    event = XINE_EVENT_VDR_CHANNELPLUS;
-    break;
-
-  case PLAYER_VDR_CHANNELMINUS:
-    event = XINE_EVENT_VDR_CHANNELMINUS;
-    break;
-
-  case PLAYER_VDR_RED:
-    event = XINE_EVENT_VDR_RED;
-    break;
-
-  case PLAYER_VDR_GREEN:
-    event = XINE_EVENT_VDR_GREEN;
-    break;
-
-  case PLAYER_VDR_YELLOW:
-    event = XINE_EVENT_VDR_YELLOW;
-    break;
-
-  case PLAYER_VDR_BLUE:
-    event = XINE_EVENT_VDR_BLUE;
-    break;
-
-  case PLAYER_VDR_PLAY:
-    event = XINE_EVENT_VDR_PLAY;
-    break;
-
-  case PLAYER_VDR_PAUSE:
-    event = XINE_EVENT_VDR_PAUSE;
-    break;
-
-  case PLAYER_VDR_STOP:
-    event = XINE_EVENT_VDR_STOP;
-    break;
-
-  case PLAYER_VDR_RECORD:
-    event = XINE_EVENT_VDR_RECORD;
-    break;
-
-  case PLAYER_VDR_FASTFWD:
-    event = XINE_EVENT_VDR_FASTFWD;
-    break;
-
-  case PLAYER_VDR_FASTREW:
-    event = XINE_EVENT_VDR_FASTREW;
-    break;
-
-  case PLAYER_VDR_POWER:
-    event = XINE_EVENT_VDR_POWER;
-    break;
-
-  case PLAYER_VDR_SCHEDULE:
-    event = XINE_EVENT_VDR_SCHEDULE;
-    break;
-
-  case PLAYER_VDR_CHANNELS:
-    event = XINE_EVENT_VDR_CHANNELS;
-    break;
-
-  case PLAYER_VDR_TIMERS:
-    event = XINE_EVENT_VDR_TIMERS;
-    break;
-
-  case PLAYER_VDR_RECORDINGS:
-    event = XINE_EVENT_VDR_RECORDINGS;
-    break;
-
-  case PLAYER_VDR_MENU:
-    event = XINE_EVENT_INPUT_MENU1;
-    break;
-
-  case PLAYER_VDR_SETUP:
-    event = XINE_EVENT_VDR_SETUP;
-    break;
-
-  case PLAYER_VDR_COMMANDS:
-    event = XINE_EVENT_VDR_COMMANDS;
-    break;
-
-  case PLAYER_VDR_0:
-    event = XINE_EVENT_INPUT_NUMBER_0;
-    break;
-
-  case PLAYER_VDR_1:
-    event = XINE_EVENT_INPUT_NUMBER_1;
-    break;
-
-  case PLAYER_VDR_2:
-    event = XINE_EVENT_INPUT_NUMBER_2;
-    break;
-
-  case PLAYER_VDR_3:
-    event = XINE_EVENT_INPUT_NUMBER_3;
-    break;
-
-  case PLAYER_VDR_4:
-    event = XINE_EVENT_INPUT_NUMBER_4;
-    break;
-
-  case PLAYER_VDR_5:
-    event = XINE_EVENT_INPUT_NUMBER_5;
-    break;
-
-  case PLAYER_VDR_6:
-    event = XINE_EVENT_INPUT_NUMBER_6;
-    break;
-
-  case PLAYER_VDR_7:
-    event = XINE_EVENT_INPUT_NUMBER_7;
-    break;
-
-  case PLAYER_VDR_8:
-    event = XINE_EVENT_INPUT_NUMBER_8;
-    break;
-
-  case PLAYER_VDR_9:
-    event = XINE_EVENT_INPUT_NUMBER_9;
-    break;
-
-  case PLAYER_VDR_USER_1:
-    event = XINE_EVENT_VDR_USER1;
-    break;
-
-  case PLAYER_VDR_USER_2:
-    event = XINE_EVENT_VDR_USER2;
-    break;
-
-  case PLAYER_VDR_USER_3:
-    event = XINE_EVENT_VDR_USER3;
-    break;
-
-  case PLAYER_VDR_USER_4:
-    event = XINE_EVENT_VDR_USER4;
-    break;
-
-  case PLAYER_VDR_USER_5:
-    event = XINE_EVENT_VDR_USER5;
-    break;
-
-  case PLAYER_VDR_USER_6:
-    event = XINE_EVENT_VDR_USER6;
-    break;
-
-  case PLAYER_VDR_USER_7:
-    event = XINE_EVENT_VDR_USER7;
-    break;
-
-  case PLAYER_VDR_USER_8:
-    event = XINE_EVENT_VDR_USER8;
-    break;
-
-  case PLAYER_VDR_USER_9:
-    event = XINE_EVENT_VDR_USER9;
-    break;
-
-  case PLAYER_VDR_VOLPLUS:
-    event = XINE_EVENT_VDR_VOLPLUS;
-    break;
-
-  case PLAYER_VDR_VOLMINUS:
-    event = XINE_EVENT_VDR_VOLMINUS;
-    break;
-
-  case PLAYER_VDR_MUTE:
-    event = XINE_EVENT_VDR_MUTE;
-    break;
-
-  case PLAYER_VDR_AUDIO:
-    event = XINE_EVENT_VDR_AUDIO;
-    break;
-
-  case PLAYER_VDR_INFO:
-    event = XINE_EVENT_VDR_INFO;
-    break;
-
-  case PLAYER_VDR_CHANNELPREVIOUS:
-    event = XINE_EVENT_VDR_CHANNELPREVIOUS;
-    break;
-
-  case PLAYER_VDR_NEXT:
-    event = XINE_EVENT_INPUT_NEXT;
-    break;
-
-  case PLAYER_VDR_PREVIOUS:
-    event = XINE_EVENT_INPUT_PREVIOUS;
-    break;
-
-  case PLAYER_VDR_SUBTITLES:
-    event = XINE_EVENT_VDR_SUBTITLES;
-    break;
-
-  default:
+  if (value >= ARRAY_NB_ELEMENTS (events))
     return;
-  }
 
-  send_event (player, event, NULL, 0);
+  send_event (player, events[value], NULL, 0);
 }
 
 /*****************************************************************************/
@@ -1437,11 +1382,16 @@ pl_supported_resources_xine (mrl_resource_t res)
 {
   switch (res)
   {
-  case MRL_RESOURCE_FILE:
   case MRL_RESOURCE_DVD:
   case MRL_RESOURCE_DVDNAV:
-  case MRL_RESOURCE_VDR:
+  case MRL_RESOURCE_FILE:
+  case MRL_RESOURCE_HTTP:
+  case MRL_RESOURCE_MMS:
   case MRL_RESOURCE_NETVDR:
+  case MRL_RESOURCE_RTP:
+  case MRL_RESOURCE_TCP:
+  case MRL_RESOURCE_UDP:
+  case MRL_RESOURCE_VDR:
     return 1;
 
   default:
@@ -1454,7 +1404,7 @@ pl_register_functions_xine (void)
 {
   player_funcs_t *funcs = NULL;
 
-  funcs = calloc (1, sizeof (player_funcs_t));
+  funcs = PCALLOC (player_funcs_t, 1);
   if (!funcs)
     return NULL;
 
@@ -1471,13 +1421,14 @@ pl_register_functions_xine (void)
   funcs->set_framedrop      = NULL;
   funcs->set_mouse_pos      = xine_player_set_mouse_pos;
   funcs->osd_show_text      = NULL;
+  funcs->osd_state          = NULL;
 
   funcs->pb_start           = xine_player_playback_start;
   funcs->pb_stop            = xine_player_playback_stop;
   funcs->pb_pause           = xine_player_playback_pause;
   funcs->pb_seek            = xine_player_playback_seek;
   funcs->pb_seek_chapter    = NULL;
-  funcs->pb_set_speed       = NULL;
+  funcs->pb_set_speed       = xine_player_playback_set_speed;
 
   funcs->audio_get_volume   = xine_player_audio_get_volume;
   funcs->audio_set_volume   = xine_player_audio_set_volume;
@@ -1488,10 +1439,9 @@ pl_register_functions_xine (void)
   funcs->audio_prev         = NULL;
   funcs->audio_next         = NULL;
 
-  funcs->video_set_fs       = NULL;
   funcs->video_set_aspect   = NULL;
   funcs->video_set_panscan  = NULL;
-  funcs->video_set_ar       = NULL;
+  funcs->video_set_ar       = xine_player_video_set_ar;
 
   funcs->sub_set_delay      = xine_player_sub_set_delay;
   funcs->sub_set_alignment  = NULL;
@@ -1514,9 +1464,9 @@ pl_register_functions_xine (void)
   funcs->tv_channel_prev    = NULL;
   funcs->tv_channel_next    = NULL;
 
-  funcs->radio_channel_set  = NULL;
-  funcs->radio_channel_prev = NULL;
-  funcs->radio_channel_next = NULL;
+  funcs->radio_channel_set  = PL_NOT_SUPPORTED;
+  funcs->radio_channel_prev = PL_NOT_SUPPORTED;
+  funcs->radio_channel_next = PL_NOT_SUPPORTED;
 
   funcs->vdr                = xine_player_vdr;
 
@@ -1528,12 +1478,9 @@ pl_register_private_xine (void)
 {
   xine_player_t *x = NULL;
 
-  x = calloc (1, sizeof (xine_player_t));
+  x = PCALLOC (xine_player_t, 1);
   if (!x)
     return NULL;
 
-  x->vo_port = PLAYER_VO_NULL;
-  x->ao_port = PLAYER_AO_NULL;
-
   return x;
 }
